@@ -1,6 +1,6 @@
 //! Benchmark the performance of the submission ring by timing tasks which
 //! submit and wait on noop requests.
-use glommio::{LocalExecutorBuilder, Task};
+use glommio::{LocalExecutorBuilder, Placement};
 use std::{
     fmt,
     time::{Duration, Instant},
@@ -59,7 +59,9 @@ fn main() {
     let num_bench_runs = 5;
     for bench in BENCH_RUNS {
         for _ in 0..num_bench_runs {
-            let ex = LocalExecutorBuilder::new().pin_to_cpu(0).make().unwrap();
+            let ex = LocalExecutorBuilder::new(Placement::Fixed(0))
+                .make()
+                .unwrap();
             let measurement = ex.run(run_bench_tasks(bench.num_tasks, bench.num_events));
 
             println!("{}", measurement);
@@ -83,7 +85,7 @@ async fn run_bench_tasks(num_tasks: u32, num_events: u32) -> Measurement {
 
     let mut handles = vec![];
     for _ in 0..num_tasks {
-        let handle = Task::local(async move {
+        let handle = glommio::spawn_local(async move {
             let start_time = Instant::now();
             let submitter = glommio::nop::NopSubmitter::new();
             for _ in 0..num_ops_per_task {
