@@ -2,12 +2,11 @@ use glommio::prelude::*;
 use std::time::Instant;
 
 fn main() {
-    let local_ex = LocalExecutorBuilder::new()
-        .pin_to_cpu(0)
+    let local_ex = LocalExecutorBuilder::new(Placement::Fixed(0))
         .spawn(|| async move {
             let mut runs = 0;
             let t = Instant::now();
-            while !Local::need_preempt() {
+            while !crate::executor().need_preempt() {
                 runs += 1;
             }
 
