@@ -5,8 +5,8 @@ use futures_lite::{
 use glommio::{
     enclose,
     net::{TcpListener, TcpStream},
-    prelude::*,
-    Task,
+    LocalExecutorBuilder,
+    Placement,
 };
 use std::{
     cell::Cell,
@@ -20,16 +20,15 @@ fn main() {
     let coord = Arc::new(Mutex::new(0));
 
     let server = coord.clone();
-    let _server_ex = LocalExecutorBuilder::new()
-        .pin_to_cpu(0)
+    let _server_ex = LocalExecutorBuilder::new(Placement::Fixed(0))
         .spawn(move || async move {
-            let bw = Local::local(async move {
+            let bw = glommio::spawn_local(async move {
                 let mut handles = Vec::new();
                 let listener = TcpListener::bind("127.0.0.1:8002").unwrap();
                 for _ in 0..101 {
                     let mut stream = listener.accept().await.unwrap();
                     handles.push(
-                        Local::local(async move {
+                        glommio::spawn_local(async move {
                             let mut byte = vec![0u8; 8192];
                             loop {
                                 let read = stream.read(&mut byte).await.unwrap();
@@ -47,7 +46,7 @@ fn main() {
             })
             .detach();
 
-            let rr = Local::local(async move {
+            let rr = glommio::spawn_local(async move {
                 let listener = TcpListener::bind("127.0.0.1:8001").unwrap();
                 let mut stream = listener.accept().await.unwrap();
 
@@ -77,8 +76,7 @@ fn main() {
 
     while *(coord.lock().unwrap()) != 1 {}
 
-    let client_ex = LocalExecutorBuilder::new()
-        .pin_to_cpu(1)
+    let client_ex = LocalExecutorBuilder::new(Placement::Fixed(1))
         .spawn(move || async move {
             let t = Instant::now();
             for _ in 0..runs {
@@ -93,7 +91,7 @@ fn main() {
             let mut handles = Vec::with_capacity(runs as usize);
             for _ in 0..100 {
                 handles.push(
-                    Task::local(async move {
+                    glommio::spawn_local(async move {
                         for _ in 0..(runs / 100) {
                             TcpStream::connect("127.0.0.1:8000").await.unwrap();
                         }
@@ -139,7 +137,7 @@ fn main() {
             let total = Rc::new(Cell::new(0.0));
             for _ in 0..100 {
                 handles.push(
-                    Task::local(enclose! { (total) async move {
+                    glommio::spawn_local(enclose! { (total) async move {
                         let t = Instant::now();
                         let mut stream = TcpStream::connect("127.0.0.1:8002").await.unwrap();
                         let byte = vec![65u8; 8192];
