@@ -14,9 +14,7 @@ use std::sync::{Arc, RwLock};
 use crate::{
     channels::shared_channel::{self, *},
     GlommioError,
-    Local,
     Result,
-    Task,
 };
 
 /// Sender side
@@ -170,7 +168,7 @@ struct Peer {
 impl Peer {
     fn new(sender: Option<SharedSender<bool>>, role: Role) -> Self {
         Self {
-            executor_id: Local::id(),
+            executor_id: crate::executor().id(),
             notifier: sender,
             role,
         }
@@ -343,7 +341,7 @@ impl<T: 'static + Send, A: MeshAdapter> MeshBuilder<T, A> {
         }
 
         let index = peers
-            .binary_search_by(|n| n.executor_id.cmp(&Local::id()))
+            .binary_search_by(|n| n.executor_id.cmp(&crate::executor().id()))
             .expect_err("Should not join a mesh more than once.");
 
         if peers.len() == self.nr_peers - 1 {
@@ -362,8 +360,7 @@ impl<T: 'static + Send, A: MeshAdapter> MeshBuilder<T, A> {
 
             let peers: Vec<_> = peers
                 .iter_mut()
-                .map(|notifier| notifier.notifier.take())
-                .flatten()
+                .filter_map(|notifier| notifier.notifier.take())
                 .collect();
 
             Ok(RegisterResult::NotificationSenders(peers))
@@ -389,7 +386,7 @@ impl<T: 'static + Send, A: MeshAdapter> MeshBuilder<T, A> {
         let (peer_id, role_id) = {
             let peers = self.peers.read().unwrap();
             let peer_id = peers
-                .binary_search_by(|n| n.executor_id.cmp(&Local::id()))
+                .binary_search_by(|n| n.executor_id.cmp(&crate::executor().id()))
                 .unwrap();
             let role_id = peers
                 .iter()
@@ -418,8 +415,8 @@ impl<T: 'static + Send, A: MeshAdapter> MeshBuilder<T, A> {
             let sender = self.channels[peer_id][i].0.take();
             let receiver = self.channels[i][peer_id].1.take();
 
-            let sender = sender.map(|sender| Task::<_>::local(sender.connect()).detach());
-            let receiver = receiver.map(|receiver| Task::<_>::local(receiver.connect()).detach());
+            let sender = sender.map(|sender| crate::spawn_local(sender.connect()).detach());
+            let receiver = receiver.map(|receiver| crate::spawn_local(receiver.connect()).detach());
 
             match sender {
                 None => {
@@ -483,16 +480,18 @@ mod tests {
         let builder2 = MeshBuilder::<usize, _>::full(nr_peers, channel_size);
 
         let executors = (0..nr_peers).map(|_| {
-            LocalExecutorBuilder::new().spawn(enclose!((builder1, builder2) move || async move {
-                let (sender1, receiver1) = builder1.join().await.unwrap();
-                assert_eq!(sender1.peer_id(), receiver1.peer_id());
+            LocalExecutorBuilder::default().spawn(
+                enclose!((builder1, builder2) move || async move {
+                    let (sender1, receiver1) = builder1.join().await.unwrap();
+                    assert_eq!(sender1.peer_id(), receiver1.peer_id());
 
-                let (sender2, receiver2) = builder2.join().await.unwrap();
-                assert_eq!(sender2.peer_id(), receiver2.peer_id());
+                    let (sender2, receiver2) = builder2.join().await.unwrap();
+                    assert_eq!(sender2.peer_id(), receiver2.peer_id());
 
-                assert_eq!(sender1.peer_id(), sender2.peer_id());
-                assert_eq!(receiver1.peer_id(), receiver2.peer_id());
-            }))
+                    assert_eq!(sender1.peer_id(), sender2.peer_id());
+                    assert_eq!(receiver1.peer_id(), receiver2.peer_id());
+                }),
+            )
         });
 
         for ex in executors.collect::<Vec<_>>() {
@@ -522,14 +521,14 @@ mod tests {
         let mesh_builder = MeshBuilder::full(nr_peers, channel_size);
 
         let executors = (0..nr_executors).map(|_| {
-            LocalExecutorBuilder::new().spawn(enclose!((mesh_builder) move || async move {
+            LocalExecutorBuilder::default().spawn(enclose!((mesh_builder) move || async move {
                 let (sender, receiver) = mesh_builder.join().await.unwrap();
                 assert_eq!(nr_peers, sender.nr_consumers());
                 assert_eq!(sender.peer_id, sender.producer_id.unwrap());
                 assert_eq!(nr_peers, receiver.nr_producers());
                 assert_eq!(receiver.peer_id, receiver.consumer_id.unwrap());
 
-                Local::local(async move {
+                crate::spawn_local(async move {
                     for peer in 0..sender.nr_consumers() {
                         if peer != sender.peer_id() {
                             sender.send_to(peer, (sender.peer_id(), peer)).await.unwrap();
@@ -557,7 +556,7 @@ mod tests {
         let mesh_builder = MeshBuilder::partial(5, 100);
 
         let producers = (0..nr_producers).map(|i| {
-            LocalExecutorBuilder::new().spawn(enclose!((mesh_builder) move || async move {
+            LocalExecutorBuilder::default().spawn(enclose!((mesh_builder) move || async move {
                 let (sender, receiver) = mesh_builder.join(Role::Producer).await.unwrap();
                 assert_eq!(nr_consumers, sender.nr_consumers());
                 assert_eq!(Some(i), sender.producer_id());
@@ -571,7 +570,7 @@ mod tests {
         });
 
         let consumers = (0..nr_consumers).map(|i| {
-            LocalExecutorBuilder::new().spawn(enclose!((mesh_builder) move || async move {
+            LocalExecutorBuilder::default().spawn(enclose!((mesh_builder) move || async move {
                 let (sender, receiver) = mesh_builder.join(Role::Consumer).await.unwrap();
                 assert_eq!(0, sender.nr_consumers());
                 assert_eq!(None, sender.producer_id());
