@@ -7,10 +7,9 @@
 use crate::{
     channels::spsc_queue::{make, BufferHalf, Consumer, Producer},
     enclose,
-    parking::Reactor,
+    reactor::Reactor,
     sys::{self, SleepNotifier},
     GlommioError,
-    Local,
     ResourceType,
 };
 use futures_lite::{future, stream::Stream};
@@ -26,7 +25,7 @@ use std::{
 type Result<T, V> = crate::Result<T, V>;
 
 /// The `SharedReceiver` is the receiving end of the Shared Channel.
-/// It implements [`Send`] so it can be passed to any thread. However
+/// It implements [`Send`] so it can be passed to any thread. However,
 /// it doesn't implement any method: before it is used it must be changed
 /// into a [`ConnectedReceiver`], which then makes sure it will be used by
 /// at most one thread.
@@ -42,7 +41,7 @@ pub struct SharedReceiver<T: Send + Sized> {
 }
 
 /// The `SharedSender` is the sending end of the Shared Channel.
-/// It implements [`Send`] so it can be passed to any thread. However
+/// It implements [`Send`] so it can be passed to any thread. However,
 /// it doesn't implement any method: before it is used it must be changed
 /// into a [`ConnectedSender`], which then makes sure it will be used by
 /// at most one thread.
@@ -144,7 +143,7 @@ impl<T: BufferHalf + Clone> Future for Connector<T> {
     }
 }
 
-/// Creates a a new `shared_channel` returning its sender and receiver
+/// Creates a new `shared_channel` returning its sender and receiver
 /// endpoints.
 ///
 /// All shared channels must be bounded.
@@ -167,7 +166,7 @@ impl<T: 'static + Send + Sized> SharedSender<T> {
     /// [`ConnectedSender`]: struct.ConnectedSender.html
     pub async fn connect(mut self) -> ConnectedSender<T> {
         let state = self.state.take().unwrap();
-        let reactor = Local::get_reactor();
+        let reactor = crate::executor().reactor();
         state.buffer.connect(reactor.id());
         let id = reactor.register_shared_channel(Box::new(enclose! {(state) move || {
             if state.buffer.consumer_disconnected() {
@@ -202,14 +201,14 @@ impl<T: Send + Sized> ConnectedSender<T> {
     /// use glommio::{channels::shared_channel, prelude::*};
     ///
     /// let (sender, receiver) = shared_channel::new_bounded(1);
-    /// let producer = LocalExecutorBuilder::new()
+    /// let producer = LocalExecutorBuilder::default()
     ///     .name("producer")
     ///     .spawn(move || async move {
     ///         let sender = sender.connect().await;
     ///         sender.try_send(0);
     ///     })
     ///     .unwrap();
-    /// let receiver = LocalExecutorBuilder::new()
+    /// let receiver = LocalExecutorBuilder::default()
     ///     .name("receiver")
     ///     .spawn(move || async move {
     ///         let mut receiver = receiver.connect().await;
@@ -233,18 +232,27 @@ impl<T: Send + Sized> ConnectedSender<T> {
         // However after we try_push(), we can still fail because the buffer
         // disconnected between now and then. That's okay as all we're trying to
         // do here is prevent unnecessary sends.
-        if self.state.buffer.consumer_disconnected() {
+        //
+        // Note that we check `producer_disconnected` because:
+        // (1) senders can be referenced by multiple tasks simultaneously
+        // (2) senders can be closed at any time using `Self::close` (which does not
+        // automatically cause the receiver / consumer to drop)
+        // (3) `Self::try_send` is used by async `Self::send`, which will not
+        // unblock correctly if the channel was closed in another task
+        if self.state.buffer.consumer_disconnected()
+            || self.state.buffer.buffer.producer_disconnected()
+        {
             return Err(GlommioError::Closed(ResourceType::Channel(item)));
         }
         match self.state.buffer.try_push(item) {
             None => {
-                if let Some(fd) = self.notifier.must_notify() {
-                    self.reactor.upgrade().unwrap().notify(fd);
-                }
+                self.notifier.notify(false);
                 Ok(())
             }
             Some(item) => {
-                let res = if self.state.buffer.consumer_disconnected() {
+                let res = if self.state.buffer.consumer_disconnected()
+                    || self.state.buffer.buffer.producer_disconnected()
+                {
                     GlommioError::Closed(ResourceType::Channel(item))
                 } else {
                     GlommioError::WouldBlock(ResourceType::Channel(item))
@@ -261,14 +269,14 @@ impl<T: Send + Sized> ConnectedSender<T> {
     /// use glommio::{channels::shared_channel, prelude::*};
     ///
     /// let (sender, receiver) = shared_channel::new_bounded(1);
-    /// let producer = LocalExecutorBuilder::new()
+    /// let producer = LocalExecutorBuilder::default()
     ///     .name("producer")
     ///     .spawn(move || async move {
     ///         let sender = sender.connect().await;
     ///         sender.send(0).await;
     ///     })
     ///     .unwrap();
-    /// let receiver = LocalExecutorBuilder::new()
+    /// let receiver = LocalExecutorBuilder::default()
     ///     .name("receiver")
     ///     .spawn(move || async move {
     ///         let mut receiver = receiver.connect().await;
@@ -290,7 +298,7 @@ impl<T: Send + Sized> ConnectedSender<T> {
     }
 
     fn wait_for_room(&self, cx: &mut Context<'_>) -> Poll<()> {
-        match self.state.buffer.free_space() > 0 {
+        match self.state.buffer.free_space() > 0 || self.state.buffer.producer_disconnected() {
             true => Poll::Ready(()),
             false => {
                 self.reactor
@@ -305,13 +313,13 @@ impl<T: Send + Sized> ConnectedSender<T> {
     /// Close the sender
     pub fn close(&self) {
         if !self.state.buffer.disconnect() {
-            if let Some(fd) = self.notifier.must_notify() {
-                if let Some(r) = self.reactor.upgrade() {
-                    r.notify(fd);
-                }
-            }
             if let Some(r) = self.reactor.upgrade() {
-                r.unregister_shared_channel(self.id)
+                self.notifier.notify(false);
+                // wake other tasks `awaiting` the same sender letting them know sender is
+                // closed; we don't `unregister_shared_channel` here because
+                // another task could still be `await`ing this sender, in which
+                // case we need to be able to `wake` it
+                r.process_shared_channels_by_id(self.id);
             }
         }
     }
@@ -323,7 +331,7 @@ impl<T: 'static + Send + Sized> SharedReceiver<T> {
     ///
     /// [`ConnectedReceiver`]: struct.ConnectedReceiver.html
     pub async fn connect(mut self) -> ConnectedReceiver<T> {
-        let reactor = Local::get_reactor();
+        let reactor = crate::executor().reactor();
         let state = self.state.take().unwrap();
         state.buffer.connect(reactor.id());
         let id = reactor.register_shared_channel(Box::new(enclose! { (state) move || {
@@ -349,28 +357,28 @@ impl<T: 'static + Send + Sized> SharedReceiver<T> {
 impl<T: Send + Sized> ConnectedReceiver<T> {
     /// Receives data from this channel
     ///
-    /// If the sender is no longer available it returns [`None`]. Otherwise
+    /// If the sender is no longer available it returns [`None`]. Otherwise,
     /// block until an item is available and returns it wrapped in [`Some`]
     ///
     /// Notice that this is also available as a Stream. Whether to consume from
     /// a stream or `recv` is up to the application. The biggest difference
     /// is that [`StreamExt`]'s [`next`] method takes a mutable reference to
     /// self. If the LocalReceiver is, say, behind an [`Rc`] it may be more
-    /// ergonomic to recv.
+    /// ergonomic to `recv`.
     ///
     /// # Examples
     /// ```
     /// use glommio::{channels::shared_channel, prelude::*};
     ///
     /// let (sender, receiver) = shared_channel::new_bounded(1);
-    /// let producer = LocalExecutorBuilder::new()
+    /// let producer = LocalExecutorBuilder::default()
     ///     .name("producer")
     ///     .spawn(move || async move {
     ///         let sender = sender.connect().await;
     ///         sender.try_send(0u32);
     ///     })
     ///     .unwrap();
-    /// let receiver = LocalExecutorBuilder::new()
+    /// let receiver = LocalExecutorBuilder::default()
     ///     .name("receiver")
     ///     .spawn(move || async move {
     ///         let mut receiver = receiver.connect().await;
@@ -414,9 +422,7 @@ impl<T: Send + Sized> ConnectedReceiver<T> {
                 }
             }
             res => {
-                if let Some(fd) = self.notifier.must_notify() {
-                    self.reactor.upgrade().unwrap().notify(fd);
-                }
+                self.notifier.notify(false);
                 Poll::Ready(res)
             }
         }
@@ -435,11 +441,10 @@ impl<T: Send + Sized> Drop for SharedSender<T> {
     fn drop(&mut self) {
         if let Some(state) = self.state.take() {
             // Never connected, we must connect ourselves.
-            state.buffer.disconnect();
-            let id = state.buffer.peer_id();
-            if let Some(notifier) = sys::get_sleep_notifier_for(id) {
-                if let Some(fd) = notifier.must_notify() {
-                    Local::get_reactor().notify(fd);
+            if !state.buffer.disconnect() {
+                let id = state.buffer.peer_id();
+                if let Some(notifier) = sys::get_sleep_notifier_for(id) {
+                    notifier.notify(false);
                 }
             }
         }
@@ -450,11 +455,10 @@ impl<T: Send + Sized> Drop for SharedReceiver<T> {
     fn drop(&mut self) {
         if let Some(state) = self.state.take() {
             // Never connected, we must connect ourselves.
-            state.buffer.disconnect();
-            let id = state.buffer.peer_id();
-            if let Some(notifier) = sys::get_sleep_notifier_for(id) {
-                if let Some(fd) = notifier.must_notify() {
-                    Local::get_reactor().notify(fd);
+            if !state.buffer.disconnect() {
+                let id = state.buffer.peer_id();
+                if let Some(notifier) = sys::get_sleep_notifier_for(id) {
+                    notifier.notify(false);
                 }
             }
         }
@@ -463,21 +467,23 @@ impl<T: Send + Sized> Drop for SharedReceiver<T> {
 
 impl<T: Send + Sized> Drop for ConnectedReceiver<T> {
     fn drop(&mut self) {
-        self.state.buffer.disconnect();
-        if let Some(fd) = self.notifier.must_notify() {
+        if !self.state.buffer.disconnect() {
             if let Some(r) = self.reactor.upgrade() {
-                r.notify(fd);
+                self.notifier.notify(false);
+                r.unregister_shared_channel(self.id);
             }
         }
-        if let Some(r) = self.reactor.upgrade() {
-            r.unregister_shared_channel(self.id)
-        }
     }
 }
 
 impl<T: Send + Sized> Drop for ConnectedSender<T> {
     fn drop(&mut self) {
-        self.close();
+        if !self.state.buffer.disconnect() {
+            if let Some(r) = self.reactor.upgrade() {
+                self.notifier.notify(false);
+                r.unregister_shared_channel(self.id)
+            }
+        }
     }
 }
 
@@ -487,6 +493,7 @@ mod test {
     use crate::{
         timer::{sleep, Timer},
         LocalExecutorBuilder,
+        Placement,
     };
     use futures_lite::{FutureExt, StreamExt};
     use std::{
@@ -501,7 +508,7 @@ mod test {
     fn producer_consumer() {
         let (sender, receiver) = new_bounded(10);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 Timer::new(Duration::from_millis(10)).await;
@@ -509,7 +516,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let x = receiver.recv().await;
@@ -525,8 +532,7 @@ mod test {
     fn producer_stream_consumer() {
         let (sender, receiver) = new_bounded(1);
 
-        let ex1 = LocalExecutorBuilder::new()
-            .pin_to_cpu(0)
+        let ex1 = LocalExecutorBuilder::new(Placement::Fixed(0))
             .spin_before_park(Duration::from_millis(1000000))
             .spawn(move || async move {
                 let sender = sender.connect().await;
@@ -537,8 +543,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
-            .pin_to_cpu(1)
+        let ex2 = LocalExecutorBuilder::new(Placement::Fixed(1))
             .spin_before_park(Duration::from_millis(1000000))
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
@@ -555,7 +560,7 @@ mod test {
     fn consumer_sleeps_before_producer_produces() {
         let (sender, receiver) = new_bounded(1);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 Timer::new(Duration::from_millis(100)).await;
                 let sender = sender.connect().await;
@@ -563,7 +568,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let recv = receiver.recv().await.unwrap();
@@ -581,7 +586,7 @@ mod test {
     fn producer_sleeps_before_consumer_consumes() {
         let (sender, receiver) = new_bounded(1);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 // This will go right away because the channel fits 1 element
@@ -591,7 +596,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 Timer::new(Duration::from_millis(100)).await;
                 let receiver = receiver.connect().await;
@@ -608,13 +613,13 @@ mod test {
     fn producer_never_connects() {
         let (sender, receiver) = new_bounded(1);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 drop(sender);
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver: ConnectedReceiver<usize> = receiver.connect().await;
                 assert!(receiver.recv().await.is_none());
@@ -629,7 +634,7 @@ mod test {
     fn destroy_with_pending_wakers() {
         let (sender, receiver) = new_bounded::<u8>(1);
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect();
                 let sender = sender.connect();
@@ -655,13 +660,13 @@ mod test {
     fn consumer_never_connects() {
         let (sender, receiver) = new_bounded(1);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 drop(receiver);
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 Timer::new(Duration::from_millis(100)).await;
                 let sender: ConnectedSender<usize> = sender.connect().await;
@@ -671,10 +676,7 @@ mod test {
                         // all good
                     }
                     Err(other_err) => {
-                        panic!(
-                            "incorrect error type: '{}' for channel send",
-                            other_err.to_string()
-                        )
+                        panic!("incorrect error type: '{}' for channel send", other_err)
                     }
                 }
             })
@@ -688,7 +690,7 @@ mod test {
     fn pass_function() {
         let (sender, receiver) = new_bounded(10);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 Timer::new(Duration::from_millis(10)).await;
@@ -698,7 +700,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let x = receiver.recv().await.unwrap();
@@ -717,7 +719,7 @@ mod test {
         let status = Arc::new(AtomicUsize::new(0));
         let s1 = status.clone();
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 sender.send(0).await.unwrap();
@@ -727,7 +729,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
 
@@ -745,7 +747,7 @@ mod test {
     fn non_copy_shared() {
         let (sender, receiver) = new_bounded(1);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 let string1 = "Some string data here..".to_string();
@@ -755,7 +757,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let x = receiver.recv().await.unwrap();
@@ -773,7 +775,7 @@ mod test {
     fn copy_shared() {
         let (sender, receiver) = new_bounded(2);
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 sender.send(100usize).await.unwrap();
@@ -781,7 +783,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let x = receiver.recv().await.unwrap();
@@ -812,7 +814,7 @@ mod test {
         let send_count = original.clone();
         let drop_count = original.clone();
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 for x in 0..1000 {
@@ -823,7 +825,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let y = receiver.recv().await.unwrap();
@@ -850,7 +852,7 @@ mod test {
         let send_count = original.clone();
         let drop_count = original.clone();
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 for x in 0..110 {
@@ -861,7 +863,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let y = receiver.recv().await.unwrap();
@@ -887,7 +889,7 @@ mod test {
         let send_count = original.clone();
         let drop_count = original.clone();
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 for x in 0..50 {
@@ -898,7 +900,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let _resp = receiver.recv().await.unwrap();
@@ -920,7 +922,7 @@ mod test {
         let send_count = original.clone();
         let drop_count = original.clone();
 
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 for x in 0..50 {
@@ -931,7 +933,7 @@ mod test {
             })
             .unwrap();
 
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 for x in 0..50 {
@@ -948,4 +950,59 @@ mod test {
         // despite differing conditions.
         assert_eq!(original.load(Ordering::Relaxed), 0usize);
     }
+
+    #[test]
+    fn close_sender_while_blocked_on_send() {
+        use std::sync::{Condvar, Mutex};
+
+        let (sender, receiver) = new_bounded(10);
+        let cv_mtx_1 = Arc::new((Condvar::new(), Mutex::new(0)));
+        let cv_mtx_2 = Arc::clone(&cv_mtx_1);
+        let cv_mtx_3 = Arc::clone(&cv_mtx_1);
+
+        let ex1 = LocalExecutorBuilder::default()
+            .spawn({
+                move || async move {
+                    let s1 = Rc::new(sender.connect().await);
+                    let s2 = Rc::clone(&s1);
+                    let t1 = crate::executor().spawn_local(async move {
+                        let mut ii = 0;
+                        while s1.try_send(ii).is_ok() {
+                            ii += 1;
+                        }
+                        s1.close();
+                        *cv_mtx_1.1.lock().unwrap() = 1;
+                        cv_mtx_1.0.notify_all();
+                    });
+                    let t2 = crate::executor().spawn_local(async move {
+                        let mut lck = cv_mtx_2
+                            .0
+                            .wait_while(cv_mtx_2.1.lock().unwrap(), |l| *l < 1)
+                            .unwrap();
+                        assert!(s2.send(-1).await.is_err());
+                        *lck = 2;
+                        cv_mtx_2.0.notify_all();
+                    });
+                    t1.await;
+                    t2.await;
+                }
+            })
+            .unwrap();
+
+        let ex2 = LocalExecutorBuilder::default()
+            .spawn(move || async move {
+                let receiver = receiver.connect().await;
+                let _lck = cv_mtx_3
+                    .0
+                    .wait_while(cv_mtx_3.1.lock().unwrap(), |l| *l < 2)
+                    .unwrap();
+                while let Some(v) = receiver.recv().await {
+                    assert!(0 <= v);
+                }
+            })
+            .unwrap();
+
+        ex1.join().unwrap();
+        ex2.join().unwrap();
+    }
 }
