@@ -61,12 +61,20 @@ pub enum QueueErrorKind {
 pub enum ReactorErrorKind {
     /// Indicates an incorrect source type.
     IncorrectSourceType,
+
+    /// Reactor unable to lock memory (max allowed, min required)
+    MemLockLimit(u64, u64),
 }
 
 impl fmt::Display for ReactorErrorKind {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             ReactorErrorKind::IncorrectSourceType => write!(f, "Incorrect source type!"),
+            ReactorErrorKind::MemLockLimit(max, min) => write!(
+                f,
+                "The memlock resource limit is too low: {} (recommended {})",
+                max, min
+            ),
         }
     }
 }
@@ -102,7 +110,12 @@ impl fmt::Display for ExecutorErrorKind {
 /// Error types that can be created when building executors.
 #[derive(Debug)]
 pub enum BuilderErrorKind {
-    /// Error type for using a [`Placement`](crate::Placement) that requires
+    /// Error type for specifying a CPU that doesn't exist
+    NonExistentCpus {
+        /// The CPU that doesn't exist
+        cpu: usize,
+    },
+    /// Error type for using a [`Placement`](crate::PoolPlacement) that requires
     /// more CPUs than available.
     InsufficientCpus {
         /// The number of CPUs required for success.
@@ -111,12 +124,12 @@ pub enum BuilderErrorKind {
         /// The number of CPUs available.
         available: usize,
     },
-    /// Error type for using [`Placement::Custom`](crate::Placement::Custom)
+    /// Error type for using [`Placement::Custom`](crate::PoolPlacement::Custom)
     /// with a number of [`CpuSet`](crate::CpuSet)s that does not match the
     /// number of shards requested.
     NrShards {
-        /// The number of [`CpuSet`](crate::CpuSet)s provided.
-        cpu_sets: usize,
+        /// The minimum number of shards that can be created
+        minimum: usize,
 
         /// The number of shards requested.
         shards: usize,
@@ -131,14 +144,17 @@ pub enum BuilderErrorKind {
 impl fmt::Display for BuilderErrorKind {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
+            Self::NonExistentCpus { cpu } => {
+                write!(f, "CPU {} doesn't exist on the host", cpu)
+            }
             Self::InsufficientCpus {
                 available,
                 required,
             } => write!(f, "found {} of {} required CPUs", available, required),
-            Self::NrShards { cpu_sets, shards } => write!(
+            Self::NrShards { minimum, shards } => write!(
                 f,
-                "provided {} CpuSet(s), requested {} shards",
-                cpu_sets, shards
+                "requested {} shards but a minimum of {} is required",
+                minimum, shards
             ),
             Self::ThreadPanic(_) => write!(f, "thread panicked"),
         }
@@ -178,7 +194,7 @@ pub enum GlommioError<T> {
     /// operation, path and file descriptor. It also contains the error
     /// from the source IO error from `std::io::*`.
     EnhancedIoError {
-        /// The source error from `std::io::Error`.
+        /// The source error from [`std::io::Error`].
         source: io::Error,
 
         /// The operation that was being attempted.
@@ -192,7 +208,7 @@ pub enum GlommioError<T> {
     },
 
     /// Executor error variant(s) for signaling certain error conditions
-    /// inside of the executor.
+    /// inside the executor.
     ExecutorError(ExecutorErrorKind),
 
     /// Error variant(s) produced when building executors.
@@ -202,6 +218,10 @@ pub enum GlommioError<T> {
     /// variant needs to return the actual item sent into the channel.
     Closed(ResourceType<T>),
 
+    /// The resource can not be closed because certain conditions are not
+    /// satisfied
+    CanNotBeClosed(ResourceType<T>, &'static str),
+
     /// Error encapsulating the `WouldBlock` error for types that don't have
     /// errors originating in the standard library. Glommio also has
     /// nonblocking types that need to indicate if they are blocking or not.
@@ -238,7 +258,7 @@ impl<T> fmt::Display for GlommioError<T> {
                 source, op, path, fd
             ),
             GlommioError::ExecutorError(err) => write!(f, "Executor error: {}", err),
-            GlommioError::BuilderError(err) => write!(f, "Executror builder error: {}", err),
+            GlommioError::BuilderError(err) => write!(f, "Executor builder error: {}", err),
             GlommioError::Closed(rt) => match rt {
                 ResourceType::Semaphore {
                     requested,
@@ -254,6 +274,11 @@ impl<T> fmt::Display for GlommioError<T> {
                 ResourceType::File(msg) => write!(f, "File is closed ({})", msg),
                 ResourceType::Gate => write!(f, "Gate is closed"),
             },
+            GlommioError::CanNotBeClosed(_, s) => write!(
+                f,
+                "Can not be closed because certain conditions are not satisfied. {}",
+                *s
+            ),
             GlommioError::WouldBlock(rt) => match rt {
                 ResourceType::Semaphore {
                     requested,
@@ -351,7 +376,7 @@ impl fmt::Display for QueueErrorKind {
 
 /// Note this is a tricky impl in the sense that you will not get the
 /// information you expect from just using this display impl on a value. On the
-/// other hand the display impl for the entire error will give correct results..
+/// other hand the display impl for the entire error will give correct results.
 impl<T> fmt::Display for ResourceType<T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let fmt_str = match self {
@@ -388,6 +413,22 @@ impl<T> Debug for GlommioError<T> {
                 ResourceType::File(msg) => write!(f, "File is closed (\"{}\")", msg),
                 ResourceType::Gate => write!(f, "Gate is closed"),
             },
+            GlommioError::CanNotBeClosed(resource, str) => match resource {
+                ResourceType::RwLock => write!(f, "RwLock can not be closed (\"{}\")", *str),
+                ResourceType::Channel(_) => write!(f, "Channel can not be closed (\"{}\")", *str),
+                ResourceType::File(msg) => {
+                    write!(f, "File can not be closed : (\"{}\"). (\"{}\")", *str, msg)
+                }
+                ResourceType::Gate => write!(f, "Gate can not be closed: {}", *str),
+                ResourceType::Semaphore {
+                    requested,
+                    available,
+                } => write!(
+                    f,
+                    "Semaphore can not be closed: {}. {{ requested {}, available: {} }}",
+                    *str, requested, available
+                ),
+            },
             GlommioError::WouldBlock(resource) => match resource {
                 ResourceType::Semaphore {
                     requested,
@@ -412,6 +453,9 @@ impl<T> Debug for GlommioError<T> {
                 }
             },
             GlommioError::BuilderError(kind) => match kind {
+                BuilderErrorKind::NonExistentCpus { cpu } => {
+                    f.write_fmt(format_args!("NonExistentCpus {{ cpu: {} }}", cpu))
+                }
                 BuilderErrorKind::InsufficientCpus {
                     required,
                     available,
@@ -419,9 +463,9 @@ impl<T> Debug for GlommioError<T> {
                     "InsufficientCpus {{ required: {}, available: {} }}",
                     required, available
                 )),
-                BuilderErrorKind::NrShards { cpu_sets, shards } => f.write_fmt(format_args!(
-                    "NrShards {{ cpu_sets: {}, shards: {} }}",
-                    cpu_sets, shards
+                BuilderErrorKind::NrShards { minimum, shards } => f.write_fmt(format_args!(
+                    "NrShards {{ minimum: {}, shards: {} }}",
+                    minimum, shards
                 )),
                 BuilderErrorKind::ThreadPanic(_) => write!(f, "Thread panicked {{ .. }}"),
             },
@@ -438,6 +482,7 @@ impl<T> Debug for GlommioError<T> {
             GlommioError::ReactorError(kind) => {
                 let kind = match kind {
                     ReactorErrorKind::IncorrectSourceType => "IncorrectSourceType",
+                    ReactorErrorKind::MemLockLimit(_, _) => "MemLockLimit",
                 };
                 write!(f, "ReactorError {{ kind: '{}' }}", kind)
             }
@@ -453,6 +498,9 @@ impl<T> From<GlommioError<T>> for io::Error {
             GlommioError::IoError(io_err) => io_err,
             GlommioError::WouldBlock(_) => io::Error::new(io::ErrorKind::WouldBlock, display_err),
             GlommioError::Closed(_) => io::Error::new(io::ErrorKind::BrokenPipe, display_err),
+            GlommioError::CanNotBeClosed(_, _) => {
+                io::Error::new(io::ErrorKind::BrokenPipe, display_err)
+            }
             GlommioError::ExecutorError(ExecutorErrorKind::QueueError { index, kind }) => {
                 match kind {
                     QueueErrorKind::StillActive => io::Error::new(
@@ -469,7 +517,8 @@ impl<T> From<GlommioError<T>> for io::Error {
                 io::ErrorKind::InvalidInput,
                 format!("invalid executor id {}", id),
             ),
-            GlommioError::BuilderError(BuilderErrorKind::InsufficientCpus { .. })
+            GlommioError::BuilderError(BuilderErrorKind::NonExistentCpus { .. })
+            | GlommioError::BuilderError(BuilderErrorKind::InsufficientCpus { .. })
             | GlommioError::BuilderError(BuilderErrorKind::NrShards { .. })
             | GlommioError::BuilderError(BuilderErrorKind::ThreadPanic(_)) => io::Error::new(
                 io::ErrorKind::Other,
@@ -491,9 +540,10 @@ impl<T> From<GlommioError<T>> for io::Error {
 
 #[cfg(test)]
 mod test {
-    use super::*;
     use std::{io, panic::panic_any};
 
+    use super::*;
+
     #[test]
     #[should_panic(
         expected = "File operation would block (\"Reading 100 bytes from position 90 would cross \
