@@ -199,14 +199,16 @@ impl BufferedFile {
 
     /// Truncates a file to the specified size.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn truncate(&self, size: u64) -> Result<()> {
         self.file.truncate(size).await.map_err(Into::into)
     }
 
     /// rename this file.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn rename<P: AsRef<Path>>(&mut self, new_path: P) -> Result<()> {
         self.file.rename(new_path).await.map_err(Into::into)
     }
@@ -217,7 +219,8 @@ impl BufferedFile {
     /// the name from the filesystem but the file will still be accessible for
     /// as long as it is open.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn remove(&self) -> Result<()> {
         self.file.remove().await.map_err(Into::into)
     }
@@ -246,7 +249,7 @@ impl BufferedFile {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::{io::dma_file::test::make_test_directories, Local};
+    use crate::io::dma_file::test::make_test_directories;
 
     macro_rules! buffered_file_test {
         ( $name:ident, $dir:ident, $kind:ident, $code:block) => {
@@ -290,7 +293,7 @@ mod test {
 
         std::assert!(path.join("testfile").exists());
 
-        let stats = Local::io_stats();
+        let stats = crate::executor().io_stats();
         assert_eq!(stats.all_rings().files_opened(), 2);
         assert_eq!(stats.all_rings().files_closed(), 2);
     });
@@ -329,7 +332,7 @@ mod test {
         writer.close().await.unwrap();
         reader.close().await.unwrap();
 
-        let stats = Local::io_stats();
+        let stats = crate::executor().io_stats();
         assert_eq!(stats.all_rings().files_opened(), 2);
         assert_eq!(stats.all_rings().files_closed(), 2);
         assert_eq!(stats.all_rings().file_buffered_reads(), (3, 15));
