@@ -5,9 +5,8 @@
 
 use crate::{
     io::{BufferedFile, ScheduledSource},
-    parking::Reactor,
+    reactor::Reactor,
     sys::{IoBuffer, Source},
-    Local,
 };
 use futures_lite::{
     io::{AsyncBufRead, AsyncRead, AsyncSeek, AsyncWrite, SeekFrom},
@@ -30,7 +29,7 @@ pin_project! {
     /// Provides linear read access to a [`BufferedFile`].
     ///
     /// The [`StreamReader`] implements [`AsyncRead`] and [`AsyncBufRead`], which can offer the user with a convenient
-    /// way of issuing reads. However note that this mandates a copy between the OS page cache and
+    /// way of issuing reads. However, note that this mandates a copy between the OS page cache and
     /// an intermediary buffer before it reaches the user-specified buffer
     ///
     /// [`BufferedFile`]: struct.BufferedFile.html
@@ -83,7 +82,7 @@ pub fn stdin() -> Stdin {
     Stdin {
         source: None,
         buffer: Buffer::new(128),
-        reactor: Rc::downgrade(&Local::get_reactor()),
+        reactor: Rc::downgrade(&crate::executor().reactor()),
     }
 }
 
@@ -144,7 +143,7 @@ struct Buffer {
 }
 
 // We can use the same implementation for reads and writes but need to
-// be careful: For writes, buffer_pos is how much we have written so
+// be careful: For writes, buffer_pos is how much we have written, so
 // we are always interested in returning 0 -> buffer_pos.
 //
 // For reads, buffer_pos means how much we have consumed, so we are
@@ -216,7 +215,7 @@ impl StreamReader {
             io_source: None,
             seek_source: None,
             buffer: Buffer::new(builder.buffer_size),
-            reactor: Rc::downgrade(&Local::get_reactor()),
+            reactor: Rc::downgrade(&crate::executor().reactor()),
         }
     }
 }
@@ -261,6 +260,7 @@ impl StreamReaderBuilder {
     /// Reads from the [`StreamReader`] will start from this position
     ///
     /// [`StreamReader`]: struct.StreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_start_pos(mut self, start: u64) -> Self {
         self.start = start;
         self
@@ -272,6 +272,7 @@ impl StreamReaderBuilder {
     /// file is larger.
     ///
     /// [`StreamReader`]: struct.StreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_end_pos(mut self, end: u64) -> Self {
         self.end = end;
         self
@@ -280,6 +281,7 @@ impl StreamReaderBuilder {
     /// Define the buffer size that will be used by the [`StreamReader`]
     ///
     /// [`StreamReader`]: struct.StreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_buffer_size(mut self, buffer_size: usize) -> Self {
         self.buffer_size = std::cmp::max(buffer_size, 1);
         self
@@ -329,9 +331,10 @@ impl StreamWriterBuilder {
         }
     }
 
-    /// Chooses whether or not to issue a sync operation when closing the file
+    /// Chooses whether to issue a sync operation when closing the file
     /// (default enabled). Disabling this is dangerous and in most cases may
     /// lead to data loss upon power failure.
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_sync_on_close_disabled(mut self, flush_disabled: bool) -> Self {
         self.sync_on_close = !flush_disabled;
         self
@@ -340,6 +343,7 @@ impl StreamWriterBuilder {
     /// Define the buffer size that will be used by the [`StreamWriter`]
     ///
     /// [`StreamWriter`]: struct.StreamWriter.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_buffer_size(mut self, buffer_size: usize) -> Self {
         self.buffer_size = std::cmp::max(buffer_size, 1);
         self
@@ -364,7 +368,7 @@ impl StreamWriter {
             file_pos: 0,
             source: None,
             buffer: Buffer::new(builder.buffer_size),
-            reactor: Rc::downgrade(&Local::get_reactor()),
+            reactor: Rc::downgrade(&crate::executor().reactor()),
         }
     }
 
@@ -382,7 +386,7 @@ impl StreamWriter {
         res.map(|_x| ())
     }
 
-    fn flush_write_buffer(&mut self, waker: Waker) -> bool {
+    fn flush_write_buffer(&mut self, waker: &Waker) -> bool {
         assert!(self.source.is_none());
         let bytes = self.buffer.consumed_bytes();
         if !bytes.is_empty() {
@@ -410,7 +414,7 @@ impl StreamWriter {
                         .upgrade()
                         .unwrap()
                         .fdatasync(self.file.as_ref().unwrap().as_raw_fd());
-                    source.add_waiter_single(cx.waker().clone());
+                    source.add_waiter_single(cx.waker());
                     self.source = Some(source);
                     Poll::Pending
                 }
@@ -430,7 +434,7 @@ impl StreamWriter {
                     .upgrade()
                     .unwrap()
                     .close(self.file.as_ref().unwrap().as_raw_fd());
-                source.add_waiter_single(cx.waker().clone());
+                source.add_waiter_single(cx.waker());
                 self.source = Some(source);
                 Poll::Pending
             }
@@ -444,7 +448,7 @@ impl StreamWriter {
 
     fn do_poll_flush(&mut self, cx: &mut Context<'_>) -> Poll<io::Result<()>> {
         match self.source.take() {
-            None => match self.flush_write_buffer(cx.waker().clone()) {
+            None => match self.flush_write_buffer(cx.waker()) {
                 true => Poll::Pending,
                 false => Poll::Ready(Ok(())),
             },
@@ -498,7 +502,7 @@ macro_rules! do_seek {
                         .upgrade()
                         .unwrap()
                         .statx($fileobj.as_raw_fd(), &$fileobj.path().unwrap());
-                    source.add_waiter_single($cx.waker().clone());
+                    source.add_waiter_single($cx.waker());
                     $source = Some(source);
                     Poll::Pending
                 }
@@ -586,7 +590,7 @@ impl AsyncBufRead for StreamReader {
                         self.buffer.max_buffer_size,
                         self.file.file.scheduler.borrow().as_ref(),
                     );
-                    source.add_waiter_single(cx.waker().clone());
+                    source.add_waiter_single(cx.waker());
                     self.io_source = Some(source);
                     Poll::Pending
                 }
@@ -612,7 +616,7 @@ impl AsyncWrite for StreamWriter {
         }
 
         if !self.buffer.data.is_empty() {
-            let x = self.flush_write_buffer(cx.waker().clone());
+            let x = self.flush_write_buffer(cx.waker());
             assert!(x);
             Poll::Pending
         } else {
@@ -671,7 +675,7 @@ impl AsyncBufRead for Stdin {
                         self.buffer.max_buffer_size,
                         None,
                     );
-                    source.add_waiter_single(cx.waker().clone());
+                    source.add_waiter_single(cx.waker());
                     self.source = Some(source);
                     Poll::Pending
                 }
