@@ -5,13 +5,23 @@
 //
 use crate::{
     io::{
-        bulk_io::{CoalescedReads, IoVec, OrderedBulkIo, ReadManyArgs, ReadManyResult},
+        bulk_io::{
+            CoalescedReads,
+            IoVec,
+            MergedBufferLimit,
+            OrderedBulkIo,
+            ReadAmplificationLimit,
+            ReadManyArgs,
+            ReadManyResult,
+        },
         glommio_file::GlommioFile,
         open_options::OpenOptions,
         read_result::ReadResult,
+        ScheduledSource,
     },
     sys::{self, sysfs, DirectIo, DmaBuffer, PollableStatus},
 };
+use futures_lite::{Stream, StreamExt};
 use nix::sys::statfs::*;
 use std::{
     cell::Ref,
@@ -23,6 +33,16 @@ use std::{
 
 pub(super) type Result<T> = crate::Result<T, ()>;
 
+/// Close result of [`DmaFile::close_rc()`]. Indicates which operation is
+/// performed on close.
+#[derive(Debug)]
+pub enum CloseResult {
+    /// The file is closed.
+    Closed,
+    /// There are other references existing, only removed this one.
+    Unreferenced,
+}
+
 pub(crate) fn align_up(v: u64, align: u64) -> u64 {
     (v + align - 1) & !(align - 1)
 }
@@ -46,11 +66,12 @@ pub(crate) fn align_down(v: u64, align: u64) -> u64 {
 pub struct DmaFile {
     file: GlommioFile,
     o_direct_alignment: u64,
+    max_sectors_size: usize,
+    max_segment_size: usize,
     pollable: PollableStatus,
 }
 
 impl DmaFile {
-    // FIXME: Don't assume 512, we can read this info from sysfs
     /// align a value up to the minimum alignment needed to access this file
     pub fn align_up(&self, v: u64) -> u64 {
         align_up(v, self.o_direct_alignment)
@@ -123,18 +144,25 @@ impl DmaFile {
             pollable = PollableStatus::Pollable;
             sys::direct_io_ify(file.as_raw_fd(), flags)?;
         }
+        let (major, minor) = (file.dev_major as usize, file.dev_minor as usize);
 
         // Docker overlay can show as dev_major 0.
         // Anything like that is obviously not something that supports the poll ring.
-        if file.dev_major == 0
-            || sysfs::BlockDevice::is_md(file.dev_major as _, file.dev_minor as _)
-        {
+        if file.dev_major == 0 || sysfs::BlockDevice::is_md(major, minor) {
             pollable = PollableStatus::NonPollable(DirectIo::Enabled);
         }
+        let max_sectors_size =
+            sysfs::BlockDevice::max_sectors_size(file.dev_major as _, file.dev_minor as _);
+        let max_segment_size =
+            sysfs::BlockDevice::max_segment_size(file.dev_major as _, file.dev_minor as _);
 
         Ok(DmaFile {
             file,
-            o_direct_alignment: 4096,
+            o_direct_alignment: (sysfs::BlockDevice::minimum_io_size(major, minor) as u64)
+                .max(sysfs::BlockDevice::logical_block_size(major, minor) as u64)
+                .max(512), // make sure the alignment is at least 512 in any case
+            max_sectors_size,
+            max_segment_size,
             pollable,
         })
     }
@@ -151,11 +179,7 @@ impl DmaFile {
             | (opts.custom_flags as libc::c_int & !libc::O_ACCMODE);
 
         let res = DmaFile::open_at(dir, path, flags, opts.mode).await;
-        let mut f = enhanced_try!(res, opdesc, Some(path), None)?;
-        // FIXME: Don't assume 512 or 4096, we can read this info from sysfs
-        // currently, we just use the minimal {values which make sense}
-        f.o_direct_alignment = if opts.write { 4096 } else { 512 };
-        Ok(f)
+        Ok(enhanced_try!(res, opdesc, Some(path), None)?)
     }
 
     pub(super) fn attach_scheduler(&self) {
@@ -186,7 +210,7 @@ impl DmaFile {
     ///
     /// It is expected that the buffer and the position be properly aligned
     /// for Direct I/O. In most platforms that means 4096 bytes. There is no
-    /// write_at_aligned, since a non aligned write would require a
+    /// write_at_aligned, since a nonaligned write would require a
     /// read-modify-write.
     ///
     /// Buffers should be allocated through [`alloc_dma_buffer`], which
@@ -277,56 +301,61 @@ impl DmaFile {
     /// This API will optimistically coalesce and deduplicate IO requests such
     /// that two overlapping or adjacent reads will result in a single IO
     /// request. This is transparent for the consumer, you will still
-    /// receive individual ReadResults corresponding to what you asked for.
+    /// receive individual [`ReadResult`]s corresponding to what you asked for.
     ///
-    /// The first argument is an iterator of [`IoVec`]. The last two
+    /// The first argument is a stream of [`IoVec`]. The last two
     /// arguments control how aggressive the IO coalescing should be:
-    /// * `max_merged_buffer_size` controls how large a merged IO request can
-    ///   be. A value of 0 disables merging completely.
-    /// * `max_read_amp` is optional and defines the maximum read amplification
-    ///   you are comfortable with. If two read requests are separated by a
-    ///   distance less than this value, they will be merged. A value `None`
-    ///   disables all read amplification limitation.
+    /// * `buffer_limit` controls how large a merged IO request can get;
+    /// * `read_amp_limit` controls how much read amplification is acceptable.
     ///
     /// It is not necessary to respect the `O_DIRECT` alignment of the file, and
-    /// this API will internally convert the positions and sizes to match.
-    pub fn read_many<V: IoVec + Unpin, S: Iterator<Item = V>>(
+    /// this API will internally align the reads appropriately.
+    pub fn read_many<V, S>(
         self: &Rc<DmaFile>,
         iovs: S,
-        max_merged_buffer_size: usize,
-        max_read_amp: Option<usize>,
-    ) -> ReadManyResult<V> {
-        let mut last: Option<(u64, usize)> = None;
-        let it = CoalescedReads::new(iovs, max_merged_buffer_size, max_read_amp)
-            .map(|iov| {
-                let eff_pos = self.align_down(iov.1.0);
-                let b = (iov.1.0 - eff_pos) as usize;
-                let eff_size = self.align_up((iov.1.1 + b) as u64) as usize;
-                (iov.0, (eff_pos, eff_size))
-            })
-            .map(|iov| {
-                let args = ReadManyArgs {
-                    user_read: iov.0,
-                    system_read: iov.1,
-                };
+        buffer_limit: MergedBufferLimit,
+        read_amp_limit: ReadAmplificationLimit,
+    ) -> ReadManyResult<V, impl Stream<Item = (ScheduledSource, ReadManyArgs<V>)>>
+    where
+        V: IoVec + Unpin,
+        S: Stream<Item = V> + Unpin,
+    {
+        let max_merged_buffer_size = match buffer_limit {
+            MergedBufferLimit::NoMerging => 0,
+            MergedBufferLimit::DeviceMaxSingleRequest => self.max_sectors_size,
+            MergedBufferLimit::Custom(limit) => {
+                self.align_down(limit.min(self.max_segment_size) as u64) as usize
+            }
+        };
 
-                if let Some(l) = &last {
-                    if l.0 == iov.1.0 && l.1 == iov.1.1 {
-                        return (None, args);
-                    }
-                }
-                let source = self.file.reactor.upgrade().unwrap().read_dma(
-                    self.as_raw_fd(),
-                    iov.1.0,
-                    iov.1.1,
-                    self.pollable,
-                    self.file.scheduler.borrow().as_ref(),
-                );
-                last = Some((iov.1.0, iov.1.1));
-                (Some(source), args)
-            });
+        let max_read_amp = match read_amp_limit {
+            ReadAmplificationLimit::NoAmplification => Some(0),
+            ReadAmplificationLimit::Custom(limit) => Some(limit),
+            ReadAmplificationLimit::NoLimit => None,
+        };
+
+        let file = self.clone();
+        let reactor = file.file.reactor.upgrade().unwrap();
+        let it = CoalescedReads::new(
+            max_merged_buffer_size,
+            max_read_amp,
+            Some(self.o_direct_alignment),
+            iovs,
+        )
+        .map(move |iov| {
+            let fd = file.as_raw_fd();
+            let pollable = file.pollable;
+            let scheduler = file.file.scheduler.borrow();
+            (
+                reactor.read_dma(fd, iov.pos(), iov.size(), pollable, scheduler.as_ref()),
+                ReadManyArgs {
+                    user_reads: iov.coalesced_user_iovecs,
+                    system_read: (iov.pos, iov.size),
+                },
+            )
+        });
         ReadManyResult {
-            inner: OrderedBulkIo::new(self.clone(), it),
+            inner: OrderedBulkIo::new(self.clone(), crate::executor().reactor().ring_depth(), it),
             current: Default::default(),
         }
     }
@@ -355,8 +384,8 @@ impl DmaFile {
     /// host the file.
     ///
     /// If the extent is larger, that means many blocks are allocated at a time.
-    /// For instance, if the extent size is 1MB, that means that only 1 out
-    /// of 4 256kB writes will be turned synchronous. Combined with diligent
+    /// For instance, if the extent size is 1 MiB, that means that only 1 out
+    /// of 4 256 KiB writes will be turned synchronous. Combined with diligent
     /// use of `fallocate` we can greatly minimize context switches.
     ///
     /// It is important not to set the extent size too big. Writes can fail
@@ -367,25 +396,28 @@ impl DmaFile {
 
     /// Truncates a file to the specified size.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn truncate(&self, size: u64) -> Result<()> {
         self.file.truncate(size).await
     }
 
-    /// rename this file.
+    /// Rename this file.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn rename<P: AsRef<Path>>(&self, new_path: P) -> Result<()> {
         self.file.rename(new_path).await
     }
 
-    /// remove this file.
+    /// Remove this file.
     ///
     /// The file does not have to be closed to be removed. Removing removes
     /// the name from the filesystem but the file will still be accessible for
     /// as long as it is open.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn remove(&self) -> Result<()> {
         self.file.remove().await
     }
@@ -406,15 +438,13 @@ impl DmaFile {
         self.file.path()
     }
 
-    /// Convenience method that closes a DmaFile wrapped inside an Rc
-    pub async fn close_rc(self: Rc<DmaFile>) -> Result<()> {
+    /// Convenience method that closes a DmaFile wrapped inside an Rc.
+    ///
+    /// Returns [CloseResult] to indicate which operation was performed.
+    pub async fn close_rc(self: Rc<DmaFile>) -> Result<CloseResult> {
         match Rc::try_unwrap(self) {
-            Err(file) => Err(io::Error::new(
-                io::ErrorKind::Other,
-                format!("{} references to file still held", Rc::strong_count(&file)),
-            )
-            .into()),
-            Ok(file) => file.close().await,
+            Err(_) => Ok(CloseResult::Unreferenced),
+            Ok(file) => file.close().await.map(|_| CloseResult::Closed),
         }
     }
 }
@@ -422,10 +452,9 @@ impl DmaFile {
 #[cfg(test)]
 pub(crate) mod test {
     use super::*;
-    use crate::{enclose, test_utils::*, ByteSliceMutExt, Latency, Local, Shares};
+    use crate::{enclose, test_utils::*, ByteSliceMutExt, Latency, Shares};
     use futures::join;
-    use futures_lite::StreamExt;
-    use itertools::Itertools;
+    use futures_lite::{stream, StreamExt};
     use rand::{seq::SliceRandom, thread_rng};
     use std::{cell::RefCell, path::PathBuf, time::Duration};
 
@@ -599,6 +628,7 @@ pub(crate) mod test {
             .expect("failed to create file");
         let read_buf = new_file.read_at(0, 500).await.expect("failed to read");
         std::assert_eq!(read_buf.len(), 500);
+        let min_read_size = new_file.align_up(500);
         for i in 0..read_buf.len() {
             std::assert_eq!(read_buf[i], 42);
         }
@@ -614,10 +644,10 @@ pub(crate) mod test {
 
         new_file.close().await.expect("failed to close file");
 
-        let stats = Local::io_stats();
+        let stats = crate::executor().io_stats();
         assert_eq!(stats.all_rings().files_opened(), 2);
         assert_eq!(stats.all_rings().files_closed(), 2);
-        assert_eq!(stats.all_rings().file_reads(), (2, 4608));
+        assert_eq!(stats.all_rings().file_reads(), (2, 4096 + min_read_size));
         assert_eq!(stats.all_rings().file_writes(), (1, 4096));
     });
 
@@ -645,7 +675,10 @@ pub(crate) mod test {
             .await
             .expect_err("pre allocating read-only files should fail");
         new_file.close().await.expect("failed to close file");
-        assert_eq!(Local::io_stats().all_rings().file_writes(), (0, 0));
+        assert_eq!(
+            crate::executor().io_stats().all_rings().file_writes(),
+            (0, 0)
+        );
     });
 
     dma_file_test!(file_empty_read, path, _k, {
@@ -658,7 +691,7 @@ pub(crate) mod test {
         std::assert_eq!(buf.len(), 0);
         new_file.close().await.expect("failed to close file");
 
-        let stats = Local::io_stats();
+        let stats = crate::executor().io_stats();
         assert_eq!(stats.all_rings().files_opened(), 1);
         assert_eq!(stats.all_rings().files_closed(), 1);
         assert_eq!(stats.all_rings().file_reads(), (1, 0));
@@ -686,7 +719,7 @@ pub(crate) mod test {
         drop(all);
         file.close().await.unwrap();
 
-        let stats = Local::io_stats();
+        let stats = crate::executor().io_stats();
         assert_eq!(stats.all_rings().files_opened(), 1);
         assert_eq!(stats.all_rings().files_closed(), 1);
         assert_eq!(stats.all_rings().file_reads(), (0, 0));
@@ -700,7 +733,7 @@ pub(crate) mod test {
         for i in 0..200 {
             let path = p.clone();
             handles.push(
-                Local::local(async move {
+                crate::spawn_local(async move {
                     let mut path = path.join("testfile");
                     path.set_extension(i.to_string());
                     let file = DmaFile::create(&path).await.expect("failed to create file");
@@ -780,36 +813,41 @@ pub(crate) mod test {
     }
 
     dma_file_test!(per_queue_stats, path, _k, {
-        let q1 = Local::create_task_queue(Shares::default(), Latency::NotImportant, "q1");
-        let q2 = Local::create_task_queue(
+        let q1 =
+            crate::executor().create_task_queue(Shares::default(), Latency::NotImportant, "q1");
+        let q2 = crate::executor().create_task_queue(
             Shares::default(),
             Latency::Matters(Duration::from_millis(1)),
             "q2",
         );
         let task1 =
-            Local::local_into(read_write(path.join("q1")), q1).expect("failed to spawn task");
+            crate::spawn_local_into(read_write(path.join("q1")), q1).expect("failed to spawn task");
         let task2 =
-            Local::local_into(read_write(path.join("q2")), q2).expect("failed to spawn task");
+            crate::spawn_local_into(read_write(path.join("q2")), q2).expect("failed to spawn task");
 
         join!(task1, task2);
 
-        let stats = Local::io_stats();
+        let stats = crate::executor().io_stats();
         assert_eq!(stats.all_rings().files_opened(), 2);
         assert_eq!(stats.all_rings().files_closed(), 2);
         assert_eq!(stats.all_rings().file_reads().0, 4);
         assert_eq!(stats.all_rings().file_writes().0, 2);
 
-        let stats = Local::task_queue_io_stats(q1).expect("failed to retrieve task queue io stats");
+        let stats = crate::executor()
+            .task_queue_io_stats(q1)
+            .expect("failed to retrieve task queue io stats");
         assert_eq!(stats.all_rings().files_opened(), 1);
         assert_eq!(stats.all_rings().files_closed(), 1);
         assert_eq!(stats.all_rings().file_reads().0, 2);
         assert_eq!(stats.all_rings().file_writes().0, 1);
 
-        let stats = Local::task_queue_io_stats(q2).expect("failed to retrieve task queue io stats");
+        let stats = crate::executor()
+            .task_queue_io_stats(q2)
+            .expect("failed to retrieve task queue io stats");
         assert_eq!(stats.all_rings().files_opened(), 1);
-        assert_eq!(stats.latency_ring.files_opened(), 1);
+        assert_eq!(stats.main_ring.files_opened(), 1);
         assert_eq!(stats.all_rings().files_closed(), 1);
-        assert_eq!(stats.latency_ring.files_closed(), 1);
+        assert_eq!(stats.main_ring.files_closed(), 1);
         assert_eq!(stats.all_rings().file_reads().0, 2);
         assert_eq!(stats.all_rings().file_writes().0, 1);
     });
@@ -817,13 +855,19 @@ pub(crate) mod test {
     dma_file_test!(file_many_reads, path, _k, {
         let new_file = Rc::new(write_dma_file(path.join("testfile"), 4096).await);
 
+        println!("{:?}", new_file);
+
         let total_reads = Rc::new(RefCell::new(0));
         let last_read = Rc::new(RefCell::new(-1));
 
-        let mut iovs = (0..512).map(|x| (x * 8, 8)).collect_vec();
+        let mut iovs: Vec<(u64, usize)> = (0..512).map(|x| (x * 8, 8)).collect();
         iovs.shuffle(&mut thread_rng());
         new_file
-            .read_many(iovs.into_iter(), 4096, None)
+            .read_many(
+                stream::iter(iovs.into_iter()),
+                MergedBufferLimit::NoMerging,
+                ReadAmplificationLimit::NoAmplification,
+            )
             .enumerate()
             .for_each(enclose! {(total_reads, last_read) |x| {
                 *total_reads.borrow_mut() += 1;
@@ -838,10 +882,9 @@ pub(crate) mod test {
             }})
             .await;
         assert_eq!(*total_reads.borrow(), 512);
-        assert_eq!(
-            Local::io_stats().all_rings().file_reads().0,
-            4096 / new_file.o_direct_alignment
-        );
+
+        let io_stats = crate::executor().io_stats().all_rings();
+        assert!(io_stats.file_reads().0 >= 1 && io_stats.file_reads().0 <= 512);
         new_file.close_rc().await.expect("failed to close file");
     });
 
@@ -851,10 +894,14 @@ pub(crate) mod test {
         let total_reads = Rc::new(RefCell::new(0));
         let last_read = Rc::new(RefCell::new(-1));
 
-        let mut iovs = (0..511).map(|x| (x * 8 + 1, 7)).collect_vec();
+        let mut iovs: Vec<(u64, usize)> = (0..511).map(|x| (x * 8 + 1, 7)).collect();
         iovs.shuffle(&mut thread_rng());
         new_file
-            .read_many(iovs.into_iter(), 4096, None)
+            .read_many(
+                stream::iter(iovs.into_iter()),
+                MergedBufferLimit::Custom(4096),
+                ReadAmplificationLimit::NoAmplification,
+            )
             .enumerate()
             .for_each(enclose! {(total_reads, last_read) |x| {
                 *total_reads.borrow_mut() += 1;
@@ -869,10 +916,8 @@ pub(crate) mod test {
             }})
             .await;
         assert_eq!(*total_reads.borrow(), 511);
-        assert_eq!(
-            Local::io_stats().all_rings().file_reads().0,
-            4096 / new_file.o_direct_alignment
-        );
+        let io_stats = crate::executor().io_stats().all_rings();
+        assert!(io_stats.file_reads().0 >= 1 && io_stats.file_reads().0 <= 512);
         new_file.close_rc().await.expect("failed to close file");
     });
 
@@ -883,7 +928,11 @@ pub(crate) mod test {
         let last_read = Rc::new(RefCell::new(-1));
 
         new_file
-            .read_many((0..511).map(|x| (x * 8 + 1, 7)), 0, Some(0))
+            .read_many(
+                stream::iter((0..511).map(|x| (x * 8 + 1, 7))),
+                MergedBufferLimit::NoMerging,
+                ReadAmplificationLimit::NoAmplification,
+            )
             .enumerate()
             .for_each(enclose! {(total_reads, last_read) |x| {
                 *total_reads.borrow_mut() += 1;
@@ -900,8 +949,14 @@ pub(crate) mod test {
             .await;
 
         assert_eq!(*total_reads.borrow(), 511);
+        let io_stats = crate::executor().io_stats().all_rings();
+        assert_eq!(io_stats.file_reads().0, 4096 / new_file.o_direct_alignment);
         assert_eq!(
-            Local::io_stats().all_rings().file_reads().0,
+            io_stats.post_reactor_io_scheduler_latency_us().count() as u64,
+            4096 / new_file.o_direct_alignment
+        );
+        assert_eq!(
+            io_stats.io_latency_us().count() as u64,
             4096 / new_file.o_direct_alignment
         );
         new_file.close_rc().await.expect("failed to close file");
