@@ -7,17 +7,14 @@ use crate::{
     io::{dma_file::align_down, read_result::ReadResult, DmaFile},
     sys::DmaBuffer,
     task,
-    ByteSliceExt,
     ByteSliceMutExt,
-    GlommioError,
-    Local,
-    ResourceType,
 };
 use ahash::AHashMap;
 use core::task::Waker;
 use futures_lite::{
     future::poll_fn,
     io::{AsyncRead, AsyncWrite},
+    ready,
     stream::{self, StreamExt},
 };
 use std::{
@@ -66,7 +63,6 @@ impl DmaStreamReaderBuilder {
     /// ```no_run
     /// use glommio::{
     ///     io::{DmaFile, DmaStreamReaderBuilder},
-    ///     Local,
     ///     LocalExecutor,
     /// };
     /// use std::rc::Rc;
@@ -77,7 +73,7 @@ impl DmaStreamReaderBuilder {
     ///     let _reader = DmaStreamReaderBuilder::from_rc(file.clone()).build();
     ///
     ///     // issue random I/O now, even though a stream is open.
-    ///     Local::local(async move {
+    ///     glommio::spawn_local(async move {
     ///         file.read_at(0, 8).await.unwrap();
     ///     })
     ///     .await;
@@ -132,6 +128,7 @@ impl DmaStreamReaderBuilder {
     /// Reads from the [`DmaStreamReader`] will start from this position
     ///
     /// [`DmaStreamReader`]: struct.DmaStreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_start_pos(mut self, start: u64) -> Self {
         self.start = start;
         self
@@ -143,6 +140,7 @@ impl DmaStreamReaderBuilder {
     /// file is larger.
     ///
     /// [`DmaStreamReader`]: struct.DmaStreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_end_pos(mut self, end: u64) -> Self {
         self.end = end;
         self
@@ -155,6 +153,7 @@ impl DmaStreamReaderBuilder {
     /// usage.
     ///
     /// [`DmaStreamReader`]: struct.DmaStreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_read_ahead(mut self, read_ahead: usize) -> Self {
         self.read_ahead = read_ahead;
         self
@@ -163,6 +162,7 @@ impl DmaStreamReaderBuilder {
     /// Define the buffer size that will be used by the [`DmaStreamReader`]
     ///
     /// [`DmaStreamReader`]: struct.DmaStreamReader.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_buffer_size(mut self, buffer_size: usize) -> Self {
         let buffer_size = std::cmp::max(buffer_size, 1);
         self.buffer_size = self.file.align_up(buffer_size as _) as usize;
@@ -221,7 +221,7 @@ impl DmaStreamReaderState {
             return;
         }
 
-        let pending = Local::local(async move {
+        let pending = crate::spawn_local(async move {
             futures_lite::future::yield_now().await;
 
             if read_state.borrow().error.is_some() {
@@ -259,38 +259,10 @@ impl DmaStreamReaderState {
         (pos & self.buffer_size_mask) >> self.buffer_size_shift
     }
 
-    fn copy_data(&mut self, pos: u64, result: &mut [u8]) -> usize {
-        let buffer_id = self.buffer_id(pos);
-        let in_buffer_offset = self.offset_of(pos);
-        if pos >= self.max_pos {
-            return 0;
-        }
-        let max_len = std::cmp::min(result.len(), (self.max_pos - pos) as usize);
-        let len: usize;
-
-        match self.buffermap.get(&buffer_id) {
-            None => {
-                panic!(
-                    "Buffer not found. But we should only call this function after we verified \
-                     that all buffers exist"
-                );
-            }
-            Some(buffer) => {
-                len = buffer.read_at(in_buffer_offset, &mut result[..max_len]);
-            }
-        }
-        len
-    }
-
-    fn add_waker(&mut self, buffer_id: u64, waker: Waker) {
-        match self.wakermap.get_mut(&buffer_id) {
-            Some(_) => {
-                panic!("More than one waker in a linear access API. Something is wrong!");
-            }
-            None => {
-                self.wakermap.insert(buffer_id, waker);
-            }
-        }
+    // returns true if heir was no waker for this buffer_id
+    // otherwise, it replaces the existing one and returns false
+    fn add_waker(&mut self, buffer_id: u64, waker: Waker) -> bool {
+        self.wakermap.insert(buffer_id, waker).is_none()
     }
 
     fn cancel_all_in_flight(&mut self) -> Vec<task::JoinHandle<()>> {
@@ -363,7 +335,7 @@ impl Drop for DmaStreamReaderState {
 /// struct.DmaStreamReader.html#method.get_buffer_aligned
 /// [`AsyncRead`]: https://docs.rs/futures/0.3.5/futures/io/trait.AsyncRead.html
 pub struct DmaStreamReader {
-    start: u64,
+    _start: u64,
     end: u64,
     current_pos: u64,
     buffer_size: u64,
@@ -410,10 +382,7 @@ impl DmaStreamReader {
         let to_cancel = handles.len();
         let cancelled = stream::iter(handles).then(|f| f).count().await;
         assert_eq!(to_cancel, cancelled);
-
-        // the file may be used by other processes so failing to close it is not an
-        // issue.
-        let _ = self.file.close_rc().await;
+        self.file.close_rc().await.unwrap();
 
         let mut state = self.state.borrow_mut();
         match state.error.take() {
@@ -447,7 +416,7 @@ impl DmaStreamReader {
 
         DmaStreamReader {
             file: builder.file,
-            start: builder.start,
+            _start: builder.start,
             end: builder.end,
             current_pos: builder.start,
             buffer_size: builder.buffer_size as _,
@@ -528,25 +497,19 @@ impl DmaStreamReader {
     }
 
     /// Allows access to the buffer that holds the current position with no
-    /// extra copy
-    //_ In order to use this API, one must guarantee that reading the specified
-    /// length may not cross into a different buffer.  Users of this API are
-    /// expected to be aware of their buffer size (selectable in the
-    /// [`DmaStreamReaderBuilder`]) and act accordingly.
+    /// extra copy.
     ///
-    /// The buffer is also not released until the returned [`ReadResult`] goes
+    /// This function returns a [`ReadResult`]. It contains all the bytes read,
+    /// which can be less than the requested amount. Users are expected to call
+    /// this in a loop like they would [`AsyncRead::poll_read`].
+    ///
+    /// The buffer is not released until the returned [`ReadResult`] goes
     /// out of scope. So if you plan to keep this alive for a long time this
     /// is probably the wrong API.
     ///
     /// If EOF is hit while reading with this method, the number of bytes in the
-    /// returned buffer will be less than number requested.
-    ///
-    /// Let's say you want to open a file and check if its header is sane: this
-    /// is a good API for that.
-    ///
-    /// But if after such header there is an index that you want to keep in
-    /// memory, then you are probably better off with one of the methods
-    /// from [`AsyncReadExt`].
+    /// returned buffer will be less than number requested and the remaining
+    /// bytes will be 0.
     ///
     /// # Examples
     /// ```no_run
@@ -567,35 +530,44 @@ impl DmaStreamReader {
     /// });
     /// ```
     ///
-    /// [`DmaStreamReader`]: struct.DmaStreamReader.html
-    /// [`DmaStreamReaderBuilder`]: struct.DmaStreamReaderBuilder.html
-    /// [`AsyncReadExt`]: https://docs.rs/futures-lite/1.11.2/futures_lite/io/trait.AsyncReadExt.html
+    /// [`AsyncRead::poll_read`]: https://docs.rs/futures-io/latest/futures_io/trait.AsyncRead.html#tymethod.poll_read
     /// [`ReadResult`]: struct.ReadResult.html
     pub async fn get_buffer_aligned(&mut self, len: u64) -> Result<ReadResult> {
-        if len == 0 {
-            return Ok(ReadResult::empty_buffer());
-        }
-
-        let (start_id, end_id, buffer_size) = {
-            let state = self.state.borrow();
-            let start_id = state.buffer_id(self.current_pos);
-            let end_id = state.buffer_id(self.current_pos + len - 1);
-            (start_id, end_id, state.buffer_size)
-        };
-
-        if start_id != end_id {
-            return Err(GlommioError::<()>::WouldBlock(ResourceType::File(format!(
-                "Reading {} bytes from position {} would cross a buffer boundary (Buffer size {})",
-                len, self.current_pos, buffer_size
-            ))));
-        }
-
-        let x = poll_fn(|cx| self.get_buffer(cx, len, start_id)).await?;
-        self.skip(len);
-        Ok(x)
+        poll_fn(|cx| self.poll_get_buffer_aligned(cx, len)).await
     }
 
-    fn get_buffer(
+    /// A variant of [`get_buffer_aligned`] that can be called from a poll
+    /// function context.
+    ///
+    /// [`get_buffer_aligned`]: Self::get_buffer_aligned
+    pub fn poll_get_buffer_aligned(
+        &mut self,
+        cx: &mut Context<'_>,
+        mut len: u64,
+    ) -> Poll<Result<ReadResult>> {
+        let (start_id, buffer_len) = {
+            let state = self.state.borrow();
+            let start_id = state.buffer_id(self.current_pos);
+            let offset = state.offset_of(self.current_pos);
+
+            // enforce max_pos
+            if self.current_pos + len > state.max_pos {
+                len = state.max_pos - self.current_pos;
+            }
+
+            (start_id, (self.buffer_size - offset as u64).min(len))
+        };
+
+        if len == 0 {
+            return Poll::Ready(Ok(ReadResult::empty_buffer()));
+        }
+
+        let x = ready!(self.poll_get_buffer(cx, buffer_len, start_id))?;
+        self.skip(x.len() as u64);
+        Poll::Ready(Ok(x))
+    }
+
+    fn poll_get_buffer(
         &mut self,
         cx: &mut Context<'_>,
         len: u64,
@@ -608,8 +580,9 @@ impl DmaStreamReader {
 
         match state.get_cached_buffer(&buffer_id).cloned() {
             None => {
-                state.fill_buffer(self.state.clone(), self.file.clone());
-                state.add_waker(buffer_id, cx.waker().clone());
+                if state.add_waker(buffer_id, cx.waker().clone()) {
+                    state.fill_buffer(self.state.clone(), self.file.clone());
+                }
                 Poll::Pending
             }
             Some(buffer) => {
@@ -634,66 +607,9 @@ impl AsyncRead for DmaStreamReader {
         cx: &mut Context<'_>,
         buf: &mut [u8],
     ) -> Poll<io::Result<usize>> {
-        let mut state = self.state.borrow_mut();
-        if let Some(err) = current_error!(state) {
-            return Poll::Ready(err);
-        }
-
-        let mut pos = self.current_pos;
-        if pos > state.max_pos {
-            return Poll::Ready(Ok(0));
-        }
-
-        let start = state.buffer_id(pos);
-        let end = state.buffer_id(pos + buf.len() as u64);
-
-        // special-casing the single buffer scenario helps small reads, as it allows
-        // us to do a single buffer lookup instead of N * 2;
-        if start == end {
-            match state.get_cached_buffer(&start).cloned() {
-                Some(buffer) => {
-                    let max_len = std::cmp::min(buf.len(), (state.max_pos - pos) as usize);
-                    let offset = state.offset_of(pos);
-                    let bytes_copied = buffer.read_at(offset, &mut buf[..max_len]);
-                    drop(state);
-                    self.skip(bytes_copied as u64);
-                    Poll::Ready(Ok(bytes_copied))
-                }
-                None => {
-                    state.fill_buffer(self.state.clone(), self.file.clone());
-                    state.add_waker(start, cx.waker().clone());
-                    Poll::Pending
-                }
-            }
-        } else {
-            for id in start..=end {
-                match state.get_cached_buffer(&id) {
-                    Some(buffer) => {
-                        if (buffer.len() as u64) < self.buffer_size {
-                            break;
-                        }
-                    }
-                    None => {
-                        state.fill_buffer(self.state.clone(), self.file.clone());
-                        state.add_waker(id, cx.waker().clone());
-                        return Poll::Pending;
-                    }
-                }
-            }
-
-            let mut current_offset = 0;
-            while current_offset < buf.len() {
-                let bytes_copied = state.copy_data(pos, &mut buf[current_offset..]);
-                current_offset += bytes_copied;
-                pos += bytes_copied as u64;
-                if bytes_copied == 0 {
-                    break;
-                }
-            }
-            drop(state);
-            self.skip(current_offset as u64);
-            Poll::Ready(Ok(current_offset))
-        }
+        let res = ready!(self.poll_get_buffer_aligned(cx, buf.len() as u64))?;
+        buf[..res.len()].copy_from_slice(&res);
+        Poll::Ready(Ok(res.len()))
     }
 }
 
@@ -751,6 +667,7 @@ impl DmaStreamWriterBuilder {
     /// writing to this [`DmaStreamWriter`] will not block.
     ///
     /// [`DmaStreamWriter`]: struct.DmaStreamWriter.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_write_behind(mut self, write_behind: usize) -> Self {
         self.write_behind = std::cmp::max(write_behind, 1);
         self
@@ -758,6 +675,7 @@ impl DmaStreamWriterBuilder {
 
     /// Does not issue a sync operation when closing the file. This is dangerous
     /// and in most cases may lead to data loss.
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_sync_on_close_disabled(mut self, flush_disabled: bool) -> Self {
         self.sync_on_close = !flush_disabled;
         self
@@ -766,6 +684,7 @@ impl DmaStreamWriterBuilder {
     /// Define the buffer size that will be used by the [`DmaStreamWriter`]
     ///
     /// [`DmaStreamWriter`]: struct.DmaStreamWriter.html
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_buffer_size(mut self, buffer_size: usize) -> Self {
         let buffer_size = std::cmp::max(buffer_size, 1);
         self.buffer_size = self.file.align_up(buffer_size as _) as usize;
@@ -932,7 +851,7 @@ impl DmaStreamWriterState {
         let final_pos = self.current_pos();
         self.flush_padded(state.clone(), file.clone());
         let mut pending = self.take_pending_handles();
-        Local::local(async move {
+        crate::spawn_local(async move {
             futures_lite::future::yield_now().await;
 
             defer! {
@@ -1021,7 +940,7 @@ impl DmaStreamWriterState {
     fn flush_one_buffer(&mut self, buffer: DmaBuffer, state: Rc<RefCell<Self>>, file: Rc<DmaFile>) {
         let aligned_pos = self.aligned_pos;
         let flush_pos = self.current_pos();
-        let handle = Local::local(async move {
+        let handle = crate::spawn_local(async move {
             let res = file.write_at(buffer, aligned_pos).await;
             let mut state = state.borrow_mut();
             if !collect_error!(state, res) {
@@ -1055,7 +974,8 @@ impl Drop for DmaStreamWriter {
         }
         if state.must_truncate() {
             let file = self.file.take().unwrap();
-            Local::get_reactor()
+            crate::executor()
+                .reactor()
                 .sys
                 .async_truncate(file.as_raw_fd(), state.flushed_pos());
         }
@@ -1460,7 +1380,7 @@ mod test {
         io::dma_file::{align_up, test::make_test_directories},
         timer::Timer,
     };
-    use futures::{AsyncReadExt, AsyncWriteExt};
+    use futures::{task::noop_waker_ref, AsyncRead, AsyncReadExt, AsyncWriteExt};
     use std::{io::ErrorKind, path::Path, time::Duration};
 
     macro_rules! file_stream_read_test {
@@ -1758,12 +1678,6 @@ mod test {
         let buffer = reader.get_buffer_aligned(8).await.unwrap();
         assert_eq!(buffer.len(), 8);
         check_contents!(*buffer, 1004);
-
-        match reader.get_buffer_aligned(20).await {
-            Err(_) => {},
-            Ok(_) => panic!("Expected an error"),
-        }
-        assert_eq!(reader.current_pos(), 1012);
         reader.skip((128 << 10) - 1012);
         let eof_short_buffer = reader.get_buffer_aligned(4).await.unwrap();
         assert_eq!(eof_short_buffer.len(), 2);
@@ -1772,6 +1686,49 @@ mod test {
         reader.close().await.unwrap();
     });
 
+    file_stream_read_test!(read_get_buffer_aligned_cross_boundaries, path, _k, file, _file_size: 16384, {
+        let mut reader = DmaStreamReaderBuilder::new(file)
+            .with_buffer_size(4096)
+            .build();
+
+        reader.skip(4094);
+
+        match reader.get_buffer_aligned(130).await {
+            Err(_) => panic!("Expected partial success"),
+            Ok(res) => {
+                assert_eq!(res.len(), 2);
+                check_contents!(*res, 4094);
+            },
+        }
+        assert_eq!(reader.current_pos(), 4096);
+
+        match reader.get_buffer_aligned(64).await {
+            Err(_) => panic!("Expected success"),
+            Ok(res) => {
+                assert_eq!(res.len(), 64);
+                check_contents!(*res, 4096);
+            },
+        }
+        assert_eq!(reader.current_pos(), 4160);
+
+        reader.skip(12160);
+
+        // EOF
+        match reader.get_buffer_aligned(128).await {
+            Err(_) => panic!("Expected success"),
+            Ok(res) => {
+                assert_eq!(res.len(), 64);
+                check_contents!(*res, 1984);
+            },
+        }
+        assert_eq!(reader.current_pos(), 16384);
+
+        let eof = reader.get_buffer_aligned(64).await.unwrap();
+        assert_eq!(eof.len(), 0);
+
+        reader.close().await.unwrap();
+    });
+
     file_stream_read_test!(read_get_buffer_aligned_zero_buffer, path, _k, file, _file_size: 131072, {
         let mut reader = DmaStreamReaderBuilder::new(file)
             .with_buffer_size(131072)
@@ -1822,18 +1779,31 @@ mod test {
     });
 
     #[track_caller]
-    fn expect_specific_error<T>(op: std::result::Result<T, io::Error>, expected_err: &'static str) {
+    fn expect_specific_error<T>(
+        op: std::result::Result<T, io::Error>,
+        expected_kind: Option<io::ErrorKind>,
+        expected_err: &'static str,
+    ) {
         match op {
             Ok(_) => panic!("should have failed"),
             Err(err) => {
-                let kind = err.kind();
-                match kind {
-                    io::ErrorKind::Other => {
-                        let x = format!("{}", err.into_inner().unwrap());
-                        assert!(x.starts_with(expected_err));
-                    }
-                    _ => panic!("Wrong error"),
+                if let Some(expected_kind) = expected_kind {
+                    assert_eq!(
+                        expected_kind,
+                        err.kind(),
+                        "expected {:?}, got {:?}",
+                        expected_kind,
+                        err.kind()
+                    );
                 }
+
+                let x = format!("{}", err.into_inner().unwrap());
+                assert!(
+                    x.starts_with(expected_err),
+                    "expected {} got {}",
+                    expected_err,
+                    x
+                );
             }
         }
     }
@@ -1846,7 +1816,21 @@ mod test {
             .build();
 
         let mut buf = [0u8; 2000];
-        expect_specific_error(reader.read_exact(&mut buf).await, "Bad file descriptor (os error 9)");
+        expect_specific_error(reader.read_exact(&mut buf).await, None, "Bad file descriptor (os error 9)");
+        reader.close().await.unwrap();
+    });
+
+    file_stream_read_test!(poll_reader_repeatedly, path, _k, file, _file_size: 131072, {
+        let mut reader = DmaStreamReaderBuilder::new(file)
+            .with_read_ahead(0)
+            .build();
+
+        let mut buf = [0u8; 2000];
+        let mut cx = std::task::Context::from_waker(noop_waker_ref());
+
+        assert!(matches!(Pin::new(&mut reader).poll_read(&mut cx, &mut buf), Poll::Pending));
+        assert!(matches!(Pin::new(&mut reader).poll_read(&mut cx, &mut buf), Poll::Pending));
+
         reader.close().await.unwrap();
     });
 
@@ -1907,7 +1891,11 @@ mod test {
             .build();
 
         writer.close().await.unwrap();
-        expect_specific_error(writer.close().await, "Bad file descriptor (os error 9)");
+        expect_specific_error(
+            writer.close().await,
+            None,
+            "Bad file descriptor (os error 9)",
+        );
     });
 
     file_stream_write_test!(write_no_write_behind, path, _k, filename, file, {
@@ -1976,7 +1964,11 @@ mod test {
             writer.write_all(&[i as u8]).await.unwrap();
         }
 
-        expect_specific_error(writer.close().await, "Bad file descriptor (os error 9)");
+        expect_specific_error(
+            writer.close().await,
+            None,
+            "Bad file descriptor (os error 9)",
+        );
     });
 
     file_stream_write_test!(flushed_position_small_buffer, path, _k, filename, file, {
@@ -2052,7 +2044,7 @@ mod test {
     #[test]
     fn flushstate() {
         test_executor!(async move {
-            let handle_gen = || Local::local(async move {}).detach();
+            let handle_gen = || crate::spawn_local(async move {}).detach();
             let mut state = DmaStreamWriterFlushState::new(4);
 
             state.on_start(8, handle_gen());
