@@ -4,6 +4,7 @@
 // This product includes software developed at Datadog (https://www.datadoghq.com/). Copyright 2020 Datadog, Inc.
 //
 use crate::io::{
+    bulk_io::{MergedBufferLimit, ReadAmplificationLimit, ReadManyArgs},
     open_options::OpenOptions,
     DmaStreamReaderBuilder,
     DmaStreamWriter,
@@ -11,9 +12,9 @@ use crate::io::{
     IoVec,
     ReadManyResult,
     ReadResult,
+    ScheduledSource,
 };
-
-use futures_lite::{future::poll_fn, io::AsyncWrite};
+use futures_lite::{future::poll_fn, io::AsyncWrite, Stream};
 use std::{
     cell::Ref,
     io,
@@ -106,6 +107,7 @@ where
     /// [`seal`]: ImmutableFilePreSealSink::seal
     /// [`build_sink`]: ImmutableFileBuilder::build_sink
     /// [`build_existing`]: ImmutableFileBuilder::build_existing
+    #[must_use = "The builder must be built to be useful"]
     pub fn new(fname: P) -> Self {
         Self {
             path: fname,
@@ -125,6 +127,7 @@ where
     /// more memory usage.
     ///
     /// [`ImmutableFile`]: ImmutableFile
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_sequential_concurrency(mut self, concurrency: usize) -> Self {
         self.concurrency = concurrency;
         self
@@ -141,6 +144,7 @@ where
     ///
     /// [`ImmutableFile`]: ImmutableFile
     /// [`ImmutableFilePreSealSink`]: ImmutableFilePreSealSink
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_sync_on_close_disabled(mut self, flush_disabled: bool) -> Self {
         self.flush_disabled = flush_disabled;
         self
@@ -150,6 +154,7 @@ where
     /// this [`ImmutableFile`]
     ///
     /// [`ImmutableFile`]: ImmutableFile
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_buffer_size(mut self, buffer_size: usize) -> Self {
         self.buffer_size = buffer_size;
         self
@@ -157,6 +162,7 @@ where
 
     /// pre-allocates space in the filesystem to hold a file at least as big as
     /// the size argument.
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_pre_allocation(mut self, size: Option<u64>) -> Self {
         self.pre_allocate = size;
         self
@@ -170,12 +176,13 @@ where
     /// host the file.
     ///
     /// If the extent is larger, that means many blocks are allocated at a time.
-    /// For instance, if the extent size is 1MB, that means that only 1 out
-    /// of 4 256kB writes will be turned synchronous. Combined with diligent
+    /// For instance, if the extent size is 1 MiB, that means that only 1 out
+    /// of 4 256 KiB writes will be turned synchronous. Combined with diligent
     /// use of `fallocate` we can greatly minimize context switches.
     ///
     /// It is important not to set the extent size too big. Writes can fail
     /// otherwise if the extent can't be allocated.
+    #[must_use = "The builder must be built to be useful"]
     pub fn with_hint_extent_size(mut self, size: Option<usize>) -> Self {
         self.hint_extent_size = size;
         self
@@ -313,7 +320,7 @@ impl ImmutableFilePreSealSink {
     /// caches upon a restart until [`ImmutableFilePreSealSink::sync`] is called
     /// (Note that [`ImmutableFilePreSealSink::seal`] implies a sync).
     ///
-    /// However within the same session, new readers trying to read from any
+    /// However, within the same session, new readers trying to read from any
     /// position before what we return in this method will be guaranteed to
     /// read the data we just wrote.
     pub fn current_flushed_pos(&self) -> u64 {
@@ -381,57 +388,54 @@ impl ImmutableFile {
     /// This API will optimistically coalesce and deduplicate IO requests such
     /// that two overlapping or adjacent reads will result in a single IO
     /// request. This is transparent for the consumer, you will still
-    /// receive individual ReadResults corresponding to what you asked for.
+    /// receive individual [`ReadResult`]s corresponding to what you asked for.
     ///
-    /// The first argument is an iterator of [`IoVec`]. The last two
+    /// The first argument is a stream of [`IoVec`]. The last two
     /// arguments control how aggressive the IO coalescing should be:
-    /// * `max_merged_buffer_size` controls how large a merged IO request can
-    ///   be. A value of 0 disables merging completely.
-    /// * `max_read_amp` is optional and defines the maximum read amplification
-    ///   you are comfortable with. If two read requests are separated by a
-    ///   distance less than this value, they will be merged. A value `None`
-    ///   disables all read amplification limitation.
+    /// * `buffer_limit` controls how large a merged IO request can get;
+    /// * `read_amp_limit` controls how much read amplification is acceptable.
     ///
     /// It is not necessary to respect the `O_DIRECT` alignment of the file, and
-    /// this API will internally convert the positions and sizes to match.
-    pub fn read_many<V, S: Iterator<Item = V>>(
+    /// this API will internally align the reads appropriately.
+    pub fn read_many<V, S>(
         &self,
         iovs: S,
-        max_merged_buffer_size: usize,
-        max_read_amp: Option<usize>,
-    ) -> ReadManyResult<V>
+        buffer_limit: MergedBufferLimit,
+        read_amp_limit: ReadAmplificationLimit,
+    ) -> ReadManyResult<V, impl Stream<Item = (ScheduledSource, ReadManyArgs<V>)>>
     where
         V: IoVec + Unpin,
-        S: Iterator<Item = V>,
+        S: Stream<Item = V> + Unpin,
     {
         self.stream_builder
             .file
-            .read_many(iovs, max_merged_buffer_size, max_read_amp)
+            .read_many(iovs, buffer_limit, read_amp_limit)
     }
 
-    /// rename this file.
+    /// Rename this file.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn rename<P: AsRef<Path>>(&self, new_path: P) -> Result<()> {
         self.stream_builder.file.rename(new_path).await
     }
 
-    /// remove this file.
+    /// Remove this file.
     ///
     /// The file does not have to be closed to be removed. Removing removes
     /// the name from the filesystem but the file will still be accessible for
     /// as long as it is open.
     ///
-    /// **Warning:** synchronous operation, will block the reactor
+    /// Note: this syscall might be issued in a background thread depending on
+    /// the system's capabilities.
     pub async fn remove(&self) -> Result<()> {
         self.stream_builder.file.remove().await
     }
 
     /// Closes this [`ImmutableFile`]
-    /// Note that this method returns an error if other entities hold references
-    /// to the underlying file, such as read streams.
     pub async fn close(self) -> Result<()> {
-        self.stream_builder.file.close_rc().await
+        self.stream_builder.file.close_rc().await?;
+        Ok(())
     }
 
     /// Creates a [`DmaStreamReaderBuilder`] from this `ImmutableFile`.
@@ -532,12 +536,12 @@ mod test {
         immutable.write(&[0, 1, 2, 3, 4, 5]).await.unwrap();
         let stream = immutable.seal().await.unwrap();
 
-        let task1 = Task::local(enclose! { (stream) async move {
+        let task1 = crate::spawn_local(enclose! { (stream) async move {
             let buf = stream.read_at(0, 6).await.unwrap();
             assert_eq!(&*buf, &[0, 1, 2, 3, 4, 5]);
         }});
 
-        let task2 = Task::local(enclose! { (stream) async move {
+        let task2 = crate::spawn_local(enclose! { (stream) async move {
             let buf = stream.read_at(0, 6).await.unwrap();
             assert_eq!(&*buf, &[0, 1, 2, 3, 4, 5]);
         }});
@@ -558,7 +562,11 @@ mod test {
 
         {
             let iovs = vec![(0, 1), (3, 1)];
-            let mut bufs = stream.read_many(iovs.into_iter(), 0, None);
+            let mut bufs = stream.read_many(
+                stream::iter(iovs.into_iter()),
+                MergedBufferLimit::NoMerging,
+                ReadAmplificationLimit::NoAmplification,
+            );
             let next_buffer = bufs.next().await.unwrap();
             assert_eq!(next_buffer.unwrap().1.len(), 1);
             let next_buffer = bufs.next().await.unwrap();
