@@ -30,7 +30,7 @@
 //! note how this works with io_uring.
 //!
 //! A timeout is submitted as an additional IO event which completes after the
-//! specified time. Therefore when you create a timeout, all that happens is
+//! specified time. Therefore, when you create a timeout, all that happens is
 //! that a completion event will appear after that specified time. This also
 //! means that when processing completion events, you need to be prepared for
 //! the possibility that the completion represents a timeout and not a normal IO
@@ -82,7 +82,7 @@ bitflags::bitflags! {
     /// [`IoUring`] initialization flags for advanced use cases.
     ///
     pub struct SetupFlags: u32 {
-        /// Poll the IO context instead of defaulting to interrupts.
+        /// Poll the IO context instead of defaulting to interrupt.
         const IOPOLL    = 1 << 0;   /* io_context is polled */
         /// Assign a kernel thread to poll the submission queue. Requires elevated privileges to set.
         const SQPOLL    = 1 << 1;   /* SQ poll thread */
@@ -161,12 +161,12 @@ impl IoUring {
     }
 
     /// Returns the `SubmissionQueue` part of the `IoUring`.
-    pub fn sq(&mut self) -> SubmissionQueue<'_> {
+    pub fn sq(&self) -> SubmissionQueue<'_> {
         SubmissionQueue::new(&*self)
     }
 
     /// Returns the `CompletionQueue` part of the `IoUring`.
-    pub fn cq(&mut self) -> CompletionQueue<'_> {
+    pub fn cq(&self) -> CompletionQueue<'_> {
         CompletionQueue::new(&*self)
     }
 
@@ -188,20 +188,6 @@ impl IoUring {
         Probe::for_ring(&mut self.ring)
     }
 
-    /// Returns the next [`SQE`] which can be prepared to submit.
-    pub fn prepare_sqe(&mut self) -> Option<SQE<'_>> {
-        unsafe { submission_queue::prepare_sqe(&mut self.ring) }
-    }
-
-    /// Returns the next `count` [`SQE`]s which can be prepared to submit as an
-    /// iterator.
-    ///
-    /// See the [`SQEs`] type for more information about how these multiple SQEs
-    /// can be used.
-    pub fn prepare_sqes(&mut self, count: u32) -> Option<SQEs<'_>> {
-        unsafe { submission_queue::prepare_sqes(&mut self.ring.sq, count) }
-    }
-
     /// Submit all prepared [`SQE`]s to the kernel.
     pub fn submit_sqes(&mut self) -> io::Result<u32> {
         self.sq().submit()
@@ -226,16 +212,7 @@ impl IoUring {
     /// Peek for any [`CQE`] that is already completed, without blocking. This
     /// will consume that CQE.
     pub fn peek_for_cqe(&mut self) -> Option<CQE> {
-        unsafe {
-            let mut cqe = MaybeUninit::uninit();
-            let count = uring_sys::io_uring_peek_batch_cqe(&mut self.ring, cqe.as_mut_ptr(), 1);
-
-            if count > 0 {
-                Some(CQE::new(NonNull::from(&self.ring), &mut *cqe.assume_init()))
-            } else {
-                None
-            }
-        }
+        self.cq().peek_for_cqe()
     }
 
     /// Block until at least one [`CQE`] is completed. This will consume that
