@@ -30,7 +30,7 @@
 //!
 //! ```
 //! use glommio::LocalExecutorBuilder;
-//! LocalExecutorBuilder::new()
+//! LocalExecutorBuilder::default()
 //!     .spawn(|| async move {
 //!         // your code here
 //!     })
@@ -49,9 +49,8 @@
 //!
 //! ```
 //! /// This will now never leave CPU 0
-//! use glommio::LocalExecutorBuilder;
-//! LocalExecutorBuilder::new()
-//!     .pin_to_cpu(0)
+//! use glommio::{LocalExecutorBuilder, Placement};
+//! LocalExecutorBuilder::new(Placement::Fixed(0))
 //!     .spawn(|| async move {
 //!         // your code here
 //!     })
@@ -77,21 +76,22 @@
 //! controllers:
 //!
 //! ```
-//! use glommio::{Latency, Local, LocalExecutorBuilder, Shares};
+//! use glommio::{executor, Latency, LocalExecutorBuilder, Placement, Shares};
 //!
-//! LocalExecutorBuilder::new()
-//!     .pin_to_cpu(0)
+//! LocalExecutorBuilder::new(Placement::Fixed(0))
 //!     .spawn(|| async move {
-//!         let tq1 = Local::create_task_queue(Shares::Static(2), Latency::NotImportant, "test1");
-//!         let tq2 = Local::create_task_queue(Shares::Static(1), Latency::NotImportant, "test2");
-//!         let t1 = Local::local_into(
+//!         let tq1 =
+//!             executor().create_task_queue(Shares::Static(2), Latency::NotImportant, "test1");
+//!         let tq2 =
+//!             executor().create_task_queue(Shares::Static(1), Latency::NotImportant, "test2");
+//!         let t1 = glommio::spawn_local_into(
 //!             async move {
 //!                 // your code here
 //!             },
 //!             tq1,
 //!         )
 //!         .unwrap();
-//!         let t2 = Local::local_into(
+//!         let t2 = glommio::spawn_local_into(
 //!             async move {
 //!                 // your code here
 //!             },
@@ -153,8 +153,8 @@
 //! a handful of corner case tasks.
 //!
 //! As atomic operations are costlier than their non-atomic counterparts, this
-//! improves efficiency by itself. However it comes with the added benefits that
-//! context switches are virtually non-existent (they only occur for kernel
+//! improves efficiency by itself. However, it comes with the added benefits
+//! that context switches are virtually non-existent (they only occur for kernel
 //! threads and interrupts) and no time is ever wasted in waiting on locks.
 //!
 //! ## Why is this a single monolith instead of many crates
@@ -164,14 +164,15 @@
 //! thread will unpark it. This allows one to effectively use that crate with
 //! very little need for anything else for the simpler cases. Combined with
 //! synchronization primitives like `Condvar`, and other thread-pool based
-//! future crates, it excels in conjunction with others but it is useful on its
+//! future crates, it excels in conjunction with others, but it is useful on its
 //! own.
 //!
 //! Now contrast that to the equivalent bits in this crate: once you `park()`
 //! the thread, you can't unpark it. I/O never gets dispatched without explicit
-//! calling into the reactor, which makes for a very weird programming model and
-//! it is very hard to integrate with the outside world since most external I/O
-//! related crates have threads that sooner or later will require `Send + Sync`.
+//! calling into the reactor, which makes for a very weird programming model,
+//! and it is very hard to integrate with the outside world since most external
+//! I/O related crates have threads that sooner or later will require [`Send`] +
+//! [`Sync`].
 //!
 //! A single crate is a way to minimize friction.
 //!
@@ -191,7 +192,7 @@
 //!    either complete or decide to yield, which means they can run for a very
 //!    long time before tasks that are latency sensitive have a chance to run.
 //!    Every time you fire a long-running operation (usually a loop) it is good
-//!    practice to check `yield_if_needed()` periodically (for example after x
+//!    practice to check [`yield_if_needed()`] periodically (for example after x
 //!    iterations of the loop). In particular, a when a new priority class is
 //!    registered, one can specify if it contains latency sensitive tasks or
 //!    not. And if the queue is marked as latency sensitive, the Latency enum
@@ -221,7 +222,7 @@
 //! *    soft    memlock        512
 //! ```
 //!
-//! To make the new limits effective, you need to login to the machine again.
+//! To make the new limits effective, you need to log in to the machine again.
 //! You can verify that the limits are updated by running the following:
 //!
 //! ```sh
@@ -238,17 +239,17 @@
 //!    here are based on the characteristics of NVMe storage. This allows us to
 //!    use io uring's poll ring for reads and writes which are interrupt free.
 //!    This also assumes that one is running either `XFS` or `Ext4` (an
-//!    assumption that Seastar also makes).
+//!    assumption that `Seastar` also makes).
 //!
 //!  - A corollary to the above is that the CPUs are likely to be the
 //!    bottleneck, so this crate has a CPU scheduler but lacks an I/O scheduler.
 //!    That, however, would be a welcome addition.
 //!
-//!  - A recent(at least 5.8) kernel is no impediment, as long as a fully
+//!  - A recent (at least 5.8) kernel is no impediment, as long as a fully
 //!    functional I/O uring is present. In fact, we require a kernel so recent
-//!    that it doesn't even exist: operations like `mkdir, ftruncate`, etc which
-//!    are not present in today's (5.8) `io_uring` are simply synchronous and
-//!    we'll live with the pain in the hopes that Linux will eventually add
+//!    that it doesn't even exist: operations like `mkdir, ftruncate`, etc.
+//!    which are not present in today's (5.8) `io_uring` are simply synchronous,
+//!    and we'll live with the pain in the hopes that Linux will eventually add
 //!    support for them.
 //!
 //! ## Missing features
@@ -261,10 +262,6 @@
 //!
 //! * As mentioned, an I/O Scheduler.
 //!
-//! * Visibility: the crate exposes no metrics on its internals, and that should
-//!   change ASAP.
-//!
-//!
 //! ## Examples
 //!
 //! Connect to `example.com:80`, or time out after 10 seconds:
@@ -289,7 +286,7 @@
 //! });
 //! ```
 #![warn(missing_docs, missing_debug_implementations, rust_2018_idioms)]
-#![cfg_attr(doc, deny(broken_intra_doc_links))]
+#![cfg_attr(doc, deny(rustdoc::broken_intra_doc_links))]
 #[macro_use]
 extern crate nix;
 extern crate alloc;
@@ -298,10 +295,7 @@ extern crate lazy_static;
 #[macro_use(defer)]
 extern crate scopeguard;
 
-use crate::parking::Reactor;
-use std::{fmt::Debug, time::Duration};
-
-/// Call `Waker::wake()` and log to `error` if panicked.
+/// Call [`Waker::wake()`] and log to `error` if panicked.
 macro_rules! wake {
     ($waker:expr $(,)?) => {
         use log::error;
@@ -319,6 +313,7 @@ mod free_list;
 #[allow(clippy::upper_case_acronyms)]
 mod iou;
 mod parking;
+mod reactor;
 mod sys;
 pub mod task;
 
@@ -330,9 +325,9 @@ mod uring_sys;
 #[doc(hidden)]
 pub mod nop;
 
-// unwraps a Result to Poll<T>: if error returns right away.
-//
-// usage is similar to future_lite::ready!
+/// Unwraps a Result to Poll<T>: if error returns right away.
+///
+/// Usage is similar to `future_lite::ready!`
 macro_rules! poll_err {
     ($e:expr $(,)?) => {
         match $e {
@@ -342,9 +337,10 @@ macro_rules! poll_err {
     };
 }
 
-// unwraps an Option to Poll<T>: if Some returns right away.
-//
-// usage is similar to future_lite::ready!
+/// Unwraps an Option to Poll<T>: if Some returns right away.
+///
+/// Usage is similar to `future_lite::ready!`
+#[allow(unused)]
 macro_rules! poll_some {
     ($e:expr $(,)?) => {
         match $e {
@@ -382,30 +378,32 @@ macro_rules! to_io_error {
 #[cfg(test)]
 macro_rules! test_executor {
     ($( $fut:expr ),+ ) => {
-    use crate::executor::{LocalExecutor, Task};
     use futures::future::join_all;
 
-    let local_ex = LocalExecutor::default();
+    let local_ex = crate::executor::LocalExecutorBuilder::new(crate::executor::Placement::Unbound)
+            .record_io_latencies(true)
+            .make()
+            .unwrap();
     local_ex.run(async move {
         let mut joins = Vec::new();
         $(
-            joins.push(Task::local($fut));
+            joins.push(crate::spawn_local($fut));
         )*
         join_all(joins).await;
     });
     }
 }
 
-// Wait for a variable to acquire a specific value.
-// The variable is expected to be a Rc<RefCell>
-//
-// Alternatively it is possible to pass a timeout in seconds
-// (through an Instant object)
-//
-// Updates to the variable gating the condition can be done (if convenient)
-// through update_cond!() (below)
-//
-// Mostly useful for tests.
+/// Wait for a variable to acquire a specific value.
+/// The variable is expected to be a Rc<RefCell>
+///
+/// Alternatively it is possible to pass a timeout in seconds
+/// (through an Instant object)
+///
+/// Updates to the variable gating the condition can be done (if convenient)
+/// through update_cond!() (below)
+///
+/// Mostly useful for tests.
 #[cfg(test)]
 macro_rules! wait_on_cond {
     ($var:expr, $val:expr) => {
@@ -413,7 +411,7 @@ macro_rules! wait_on_cond {
             if *($var.borrow()) == $val {
                 break;
             }
-            Task::<()>::later().await;
+            crate::executor().yield_task_queue_now().await;
         }
     };
     ($var:expr, $val:expr, $instantval:expr) => {
@@ -426,7 +424,7 @@ macro_rules! wait_on_cond {
             if start.elapsed().as_secs() > $instantval {
                 panic!("test timed out");
             }
-            Task::<()>::later().await;
+            crate::executor().yield_task_queue_now().await;
         }
     };
 }
@@ -465,6 +463,7 @@ mod shares;
 pub mod sync;
 pub mod timer;
 
+use crate::reactor::Reactor;
 pub use crate::{
     byte_slice_ext::{ByteSliceExt, ByteSliceMutExt},
     error::{
@@ -477,12 +476,24 @@ pub use crate::{
         Result,
     },
     executor::{
+        allocate_dma_buffer,
+        allocate_dma_buffer_global,
+        executor,
+        spawn_local,
+        spawn_local_into,
+        spawn_scoped_local,
+        spawn_scoped_local_into,
+        stall::{DefaultStallDetectionHandler, StallDetectionHandler},
+        yield_if_needed,
         CpuSet,
+        ExecutorJoinHandle,
+        ExecutorProxy,
         ExecutorStats,
         LocalExecutor,
         LocalExecutorBuilder,
         LocalExecutorPoolBuilder,
         Placement,
+        PoolPlacement,
         PoolThreadHandles,
         ScopedTask,
         Task,
@@ -494,22 +505,32 @@ pub use crate::{
 };
 pub use enclose::enclose;
 pub use scopeguard::defer;
-use std::iter::Sum;
+use sketches_ddsketch::DDSketch;
+use std::{
+    fmt::{Debug, Formatter},
+    iter::Sum,
+    time::Duration,
+};
 
 /// Provides common imports that almost all Glommio applications will need
 pub mod prelude {
     #[doc(no_inline)]
     pub use crate::{
         error::GlommioError,
+        executor,
+        spawn_local,
+        spawn_local_into,
+        yield_if_needed,
         ByteSliceExt,
         ByteSliceMutExt,
+        ExecutorProxy,
         IoStats,
         Latency,
-        Local,
         LocalExecutor,
         LocalExecutorBuilder,
         LocalExecutorPoolBuilder,
         Placement,
+        PoolPlacement,
         PoolThreadHandles,
         RingIoStats,
         Shares,
@@ -517,20 +538,9 @@ pub mod prelude {
     };
 }
 
-/// Local is an ergonomic way to access the local executor.
-/// The local is executed through a Task type, but the Task type has a type
-/// parameter consisting of the return type of the future encapsulated by this
-/// task.
-///
-/// However for associated functions without a self parameter, like `local()`
-/// and `local_into()`, the type is always `()` and Rust is not able to elide.
-///
-/// Writing `Task::<()>::function()` works, but it is not very ergonomic.
-pub type Local = Task<()>;
-
 /// An attribute of a [`TaskQueue`], passed during its creation.
 ///
-/// This tells the executor whether or not tasks in this class are latency
+/// This tells the executor whether tasks in this class are latency
 /// sensitive. Latency sensitive tasks will be placed in their own I/O ring,
 /// and tasks in background classes can cooperatively preempt themselves in
 /// the faces of pending events for latency classes.
@@ -539,7 +549,8 @@ pub type Local = Task<()>;
 #[derive(Clone, Copy, Debug)]
 pub enum Latency {
     /// Tasks marked as `Latency::Matters` will cooperatively signal to other
-    /// tasks that they should preempt often
+    /// tasks that they should preempt often. The `Duration` argument
+    /// contributes to the rate of preemption of the scheduler.
     Matters(Duration),
 
     /// Tasks marked as `Latency::NotImportant` will not signal to other tasks
@@ -550,14 +561,14 @@ pub enum Latency {
 #[derive(Clone, Copy, Debug)]
 pub(crate) struct IoRequirements {
     latency_req: Latency,
-    io_handle: usize,
+    _io_handle: usize,
 }
 
 impl Default for IoRequirements {
     fn default() -> Self {
         Self {
             latency_req: Latency::NotImportant,
-            io_handle: 0,
+            _io_handle: 0,
         }
     }
 }
@@ -566,14 +577,15 @@ impl IoRequirements {
     fn new(latency: Latency, handle: usize) -> Self {
         Self {
             latency_req: latency,
-            io_handle: handle,
+            _io_handle: handle,
         }
     }
 }
 
 /// Stores information about IO performed in a specific ring
-#[derive(Default, Debug, Copy, Clone)]
+#[derive(Clone)]
 pub struct RingIoStats {
+    // Counters
     pub(crate) files_opened: u64,
     pub(crate) files_closed: u64,
     pub(crate) file_reads: u64,
@@ -586,13 +598,64 @@ pub struct RingIoStats {
     pub(crate) file_bytes_written: u64,
     pub(crate) file_buffered_writes: u64,
     pub(crate) file_buffered_bytes_written: u64,
+
+    // Distributions
+    pub(crate) pre_reactor_io_scheduler_latency_us: sketches_ddsketch::DDSketch,
+    pub(crate) io_latency_us: sketches_ddsketch::DDSketch,
+    pub(crate) post_reactor_io_scheduler_latency_us: sketches_ddsketch::DDSketch,
+}
+
+impl Default for RingIoStats {
+    fn default() -> Self {
+        Self {
+            files_opened: 0,
+            files_closed: 0,
+            file_reads: 0,
+            file_bytes_read: 0,
+            file_buffered_reads: 0,
+            file_buffered_bytes_read: 0,
+            file_deduped_reads: 0,
+            file_deduped_bytes_read: 0,
+            file_writes: 0,
+            file_bytes_written: 0,
+            file_buffered_writes: 0,
+            file_buffered_bytes_written: 0,
+            pre_reactor_io_scheduler_latency_us: sketches_ddsketch::DDSketch::new(
+                sketches_ddsketch::Config::new(0.01, 2048, 1.0e-9),
+            ),
+            io_latency_us: sketches_ddsketch::DDSketch::new(sketches_ddsketch::Config::new(
+                0.01, 2048, 1.0e-9,
+            )),
+            post_reactor_io_scheduler_latency_us: sketches_ddsketch::DDSketch::new(
+                sketches_ddsketch::Config::new(0.01, 2048, 1.0e-9),
+            ),
+        }
+    }
+}
+
+impl Debug for RingIoStats {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("RingIoStats")
+            .field("files_opened", &self.files_opened)
+            .field("files_closed", &self.files_closed)
+            .field("file_reads", &self.file_reads)
+            .field("file_bytes_read", &self.file_bytes_read)
+            .field("file_buffered_reads", &self.file_buffered_reads)
+            .field("file_buffered_bytes_read", &self.file_buffered_bytes_read)
+            .field("file_deduped_reads", &self.file_deduped_reads)
+            .field("file_deduped_bytes_read", &self.file_deduped_bytes_read)
+            .field("file_writes", &self.file_writes)
+            .field("file_bytes_written", &self.file_bytes_written)
+            .field("file_buffered_writes", &self.file_buffered_writes)
+            .field(
+                "file_buffered_bytes_written",
+                &self.file_buffered_bytes_written,
+            )
+            .finish_non_exhaustive()
+    }
 }
 
 impl RingIoStats {
-    fn new() -> Self {
-        Default::default()
-    }
-
     /// The total amount of files opened in this executor so far.
     ///
     /// [`files_opened`] - [`files_closed`] gives the current open files count
@@ -648,24 +711,57 @@ impl RingIoStats {
     pub fn file_buffered_writes(&self) -> (u64, u64) {
         (self.file_buffered_writes, self.file_buffered_bytes_written)
     }
+
+    /// The pre-reactor IO scheduler latency
+    ///
+    /// Returns a distribution of measures tracking the time between the moment
+    /// an IO operation was queued up and the moment it was submitted to the
+    /// kernel
+    pub fn pre_reactor_io_scheduler_latency_us(&self) -> &DDSketch {
+        &self.pre_reactor_io_scheduler_latency_us
+    }
+
+    /// The IO latency
+    ///
+    /// Returns a distribution of measures tracking the time sources spent in
+    /// the ring
+    pub fn io_latency_us(&self) -> &DDSketch {
+        &self.io_latency_us
+    }
+
+    /// The post-reactor IO scheduler latency
+    ///
+    /// Returns a distribution of measures tracking the time between the moment
+    /// an IO operation was marked as fulfilled by the reactor and when the
+    /// result was consumed by the application code.
+    pub fn post_reactor_io_scheduler_latency_us(&self) -> &DDSketch {
+        &self.post_reactor_io_scheduler_latency_us
+    }
 }
 
-impl Sum<RingIoStats> for RingIoStats {
-    fn sum<I: Iterator<Item = RingIoStats>>(iter: I) -> Self {
-        iter.fold(RingIoStats::new(), |a, b| RingIoStats {
-            files_opened: a.files_opened + b.files_opened,
-            files_closed: a.files_closed + b.files_closed,
-            file_reads: a.file_reads + b.file_reads,
-            file_bytes_read: a.file_bytes_read + b.file_bytes_read,
-            file_buffered_reads: a.file_buffered_reads + b.file_buffered_reads,
-            file_buffered_bytes_read: a.file_buffered_bytes_read + b.file_buffered_bytes_read,
-            file_deduped_reads: a.file_deduped_reads + b.file_deduped_reads,
-            file_deduped_bytes_read: a.file_deduped_bytes_read + b.file_deduped_bytes_read,
-            file_writes: a.file_writes + b.file_writes,
-            file_bytes_written: a.file_bytes_written + b.file_bytes_written,
-            file_buffered_writes: a.file_buffered_writes + b.file_buffered_writes,
-            file_buffered_bytes_written: a.file_buffered_bytes_written
-                + b.file_buffered_bytes_written,
+impl<'a> Sum<&'a RingIoStats> for RingIoStats {
+    fn sum<I: Iterator<Item = &'a RingIoStats>>(iter: I) -> Self {
+        iter.fold(RingIoStats::default(), |mut a, b| {
+            a.files_opened += b.files_opened;
+            a.files_closed += b.files_closed;
+            a.file_reads += b.file_reads;
+            a.file_bytes_read += b.file_bytes_read;
+            a.file_buffered_reads += b.file_buffered_reads;
+            a.file_buffered_bytes_read += b.file_buffered_bytes_read;
+            a.file_deduped_reads += b.file_deduped_reads;
+            a.file_deduped_bytes_read += b.file_deduped_bytes_read;
+            a.file_writes += b.file_writes;
+            a.file_bytes_written += b.file_bytes_written;
+            a.file_buffered_writes += b.file_buffered_writes;
+            a.file_buffered_bytes_written += b.file_buffered_bytes_written;
+            a.pre_reactor_io_scheduler_latency_us
+                .merge(&b.pre_reactor_io_scheduler_latency_us)
+                .unwrap();
+            a.io_latency_us.merge(&b.io_latency_us).unwrap();
+            a.post_reactor_io_scheduler_latency_us
+                .merge(&b.post_reactor_io_scheduler_latency_us)
+                .unwrap();
+            a
         })
     }
 }
@@ -692,7 +788,7 @@ impl IoStats {
 
     /// Combine stats from all rings
     pub fn all_rings(&self) -> RingIoStats {
-        [self.main_ring, self.latency_ring, self.poll_ring]
+        [&self.main_ring, &self.latency_ring, &self.poll_ring]
             .iter()
             .copied()
             .sum()
