@@ -4,11 +4,17 @@
 // This product includes software developed at Datadog (https://www.datadoghq.com/). Copyright 2020 Datadog, Inc.
 //
 use super::stream::GlommioStream;
-use crate::{net::yolo_accept, parking::Reactor, GlommioError, Local};
+use crate::{
+    net::{
+        stream::{Buffered, NonBuffered, Preallocated, RxBuf},
+        yolo_accept,
+    },
+    reactor::Reactor,
+    GlommioError,
+};
 use futures_lite::{
     future::poll_fn,
     io::{AsyncBufRead, AsyncRead, AsyncWrite},
-    ready,
     stream::{self, Stream},
 };
 use nix::sys::socket::{InetAddr, SockAddr};
@@ -71,6 +77,19 @@ pub struct TcpListener {
     listener: net::TcpListener,
 }
 
+impl FromRawFd for TcpListener {
+    /// Convert an already bound and listening RawFd into a TcpListener
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        let sk = Socket::from_raw_fd(fd);
+        let listener = sk.into_tcp_listener();
+
+        TcpListener {
+            reactor: Rc::downgrade(&crate::executor().reactor()),
+            listener,
+        }
+    }
+}
+
 impl TcpListener {
     /// Creates a TCP listener bound to the specified address.
     ///
@@ -111,7 +130,7 @@ impl TcpListener {
         let listener = sk.into_tcp_listener();
 
         Ok(TcpListener {
-            reactor: Rc::downgrade(&Local::get_reactor()),
+            reactor: Rc::downgrade(&crate::executor().reactor()),
             listener,
         })
     }
@@ -321,7 +340,7 @@ impl AcceptedTcpStream {
     ///     let accepted = listener.shared_accept().await.unwrap();
     ///     sender.try_send(accepted).unwrap();
     ///
-    ///     let ex1 = LocalExecutorBuilder::new()
+    ///     let ex1 = LocalExecutorBuilder::default()
     ///         .spawn(move || async move {
     ///             let receiver = receiver.connect().await;
     ///             let accepted = receiver.recv().await.unwrap();
@@ -348,22 +367,22 @@ pin_project! {
     /// [`AsyncBufRead`]: https://docs.rs/futures-io/0.3.8/futures_io/trait.AsyncBufRead.html
     /// [`AsyncWrite`]: https://docs.rs/futures-io/0.3.8/futures_io/trait.AsyncWrite.html
 
-    pub struct TcpStream {
-        stream: GlommioStream<net::TcpStream>
+    pub struct TcpStream<B: RxBuf = NonBuffered> {
+        stream: GlommioStream<net::TcpStream, B>
     }
 }
 
 impl From<socket2::Socket> for TcpStream {
     fn from(socket: socket2::Socket) -> TcpStream {
         Self {
-            stream: GlommioStream::<net::TcpStream>::from(socket),
+            stream: GlommioStream::from(socket),
         }
     }
 }
 
-impl AsRawFd for TcpStream {
+impl<B: RxBuf> AsRawFd for TcpStream<B> {
     fn as_raw_fd(&self) -> RawFd {
-        self.stream.as_raw_fd()
+        self.stream.stream().as_raw_fd()
     }
 }
 
@@ -402,7 +421,7 @@ impl TcpStream {
     pub async fn connect<A: ToSocketAddrs>(addr: A) -> Result<TcpStream> {
         let addr = addr.to_socket_addrs()?.next().unwrap();
         let (addr, socket) = make_tcp_socket(&addr)?;
-        let reactor = Local::get_reactor();
+        let reactor = crate::executor().reactor();
         let source = reactor.connect(socket.as_raw_fd(), addr);
         source.collect_rw().await?;
 
@@ -411,6 +430,75 @@ impl TcpStream {
         })
     }
 
+    /// Creates a TCP connection to the specified address with a timeout.
+    ///
+    /// It is an error to pass a zero `Duration` to this function.
+    ///
+    /// Timeouts are implemented using `io_uring`'s `IORING_OP_LINK_TIMEOUT`.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// use glommio::{net::TcpStream, LocalExecutor};
+    ///
+    /// use std::time::Duration;
+    ///
+    /// let ex = LocalExecutor::default();
+    /// ex.run(async move {
+    ///     TcpStream::connect_timeout("127.0.0.1:10000", Duration::from_secs(10))
+    ///         .await
+    ///         .unwrap();
+    /// })
+    /// ```
+    pub async fn connect_timeout<A: ToSocketAddrs>(
+        addr: A,
+        duration: Duration,
+    ) -> Result<TcpStream> {
+        if duration.as_secs() == 0 && duration.subsec_nanos() == 0 {
+            return Err(io::Error::new(
+                io::ErrorKind::InvalidInput,
+                "cannot set a 0 duration timeout",
+            )
+            .into());
+        }
+
+        let addr = addr.to_socket_addrs()?.next().unwrap();
+        let (addr, socket) = make_tcp_socket(&addr)?;
+        let reactor = crate::executor().reactor();
+        let source = reactor.connect_timeout(socket.as_raw_fd(), addr, duration);
+
+        // connect_timeout submits two sqes to io_uring: a connect sqe soft-linked
+        // with a LINK_TIMEOUT sqe. If the timeout fires, the connect sqe fails with
+        // ECANCELED. We map that error to TimedOut to match the standard library's API.
+        source
+            .collect_rw()
+            .await
+            .map_err(|err| match err.raw_os_error() {
+                Some(libc::ECANCELED) => {
+                    io::Error::new(io::ErrorKind::TimedOut, "connection timed out")
+                }
+                _ => err,
+            })?;
+
+        Ok(TcpStream {
+            stream: GlommioStream::from(socket),
+        })
+    }
+
+    /// Creates a buffered TCP connection with default receive buffer.
+    pub fn buffered(self) -> TcpStream<Preallocated> {
+        self.buffered_with(Preallocated::default())
+    }
+
+    /// Creates a buffered TCP connection with custom receive buffer.
+    pub fn buffered_with<B: Buffered>(self, buf: B) -> TcpStream<B> {
+        TcpStream {
+            stream: self.stream.buffered_with(buf),
+        }
+    }
+}
+
+impl<B: RxBuf> TcpStream<B> {
     /// Sets the read timeout to the timeout specified.
     ///
     /// If the value specified is [`None`], then read calls will block
@@ -465,61 +553,6 @@ impl TcpStream {
         self.stream.write_timeout()
     }
 
-    /// Creates a TCP connection to the specified address with a timeout.
-    ///
-    /// It is an error to pass a zero `Duration` to this function.
-    ///
-    /// Timeouts are implemented using `io_uring`'s `IORING_OP_LINK_TIMEOUT`.
-    ///
-    /// # Examples
-    ///
-    /// ```no_run
-    /// use glommio::{net::TcpStream, LocalExecutor};
-    ///
-    /// use std::time::Duration;
-    ///
-    /// let ex = LocalExecutor::default();
-    /// ex.run(async move {
-    ///     TcpStream::connect_timeout("127.0.0.1:10000", Duration::from_secs(10))
-    ///         .await
-    ///         .unwrap();
-    /// })
-    /// ```
-    pub async fn connect_timeout<A: ToSocketAddrs>(
-        addr: A,
-        duration: Duration,
-    ) -> Result<TcpStream> {
-        if duration.as_secs() == 0 && duration.subsec_nanos() == 0 {
-            return Err(io::Error::new(
-                io::ErrorKind::InvalidInput,
-                "cannot set a 0 duration timeout",
-            )
-            .into());
-        }
-
-        let addr = addr.to_socket_addrs()?.next().unwrap();
-        let (addr, socket) = make_tcp_socket(&addr)?;
-        let reactor = Local::get_reactor();
-        let source = reactor.connect_timeout(socket.as_raw_fd(), addr, duration);
-
-        // connect_timeout submits two sqes to io_uring: a connect sqe soft-linked
-        // with a LINK_TIMEOUT sqe. If the timeout fires, the connect sqe fails with
-        // ECANCELED. We map that error to TimedOut to match the standard library's API.
-        source
-            .collect_rw()
-            .await
-            .map_err(|err| match err.raw_os_error() {
-                Some(libc::ECANCELED) => {
-                    io::Error::new(io::ErrorKind::TimedOut, "connection timed out")
-                }
-                _ => err,
-            })?;
-
-        Ok(TcpStream {
-            stream: GlommioStream::from(socket),
-        })
-    }
-
     /// Shuts down the read, write, or both halves of this connection.
     pub async fn shutdown(&self, how: Shutdown) -> Result<()> {
         poll_fn(|cx| self.stream.poll_shutdown(cx, how))
@@ -547,7 +580,7 @@ impl TcpStream {
     /// });
     /// ```
     pub fn set_nodelay(&self, value: bool) -> Result<()> {
-        self.stream.stream.set_nodelay(value).map_err(Into::into)
+        self.stream.stream().set_nodelay(value).map_err(Into::into)
     }
 
     /// Gets the `TCP_NODELAY` option on this socket.
@@ -567,17 +600,7 @@ impl TcpStream {
     /// });
     /// ```
     pub fn nodelay(&self) -> Result<bool> {
-        self.stream.stream.nodelay().map_err(Into::into)
-    }
-
-    /// Sets the buffer size used on the receive path
-    pub fn set_buffer_size(&mut self, buffer_size: usize) {
-        self.stream.rx_buf_size = buffer_size;
-    }
-
-    /// gets the buffer size used
-    pub fn buffer_size(&mut self) -> usize {
-        self.stream.rx_buf_size
+        self.stream.stream().nodelay().map_err(Into::into)
     }
 
     /// Gets the value of the `IP_TTL` option for this socket.
@@ -597,7 +620,7 @@ impl TcpStream {
     /// });
     /// ```
     pub fn ttl(&self) -> Result<u32> {
-        Ok(self.stream.stream.ttl()?)
+        Ok(self.stream.stream().ttl()?)
     }
 
     /// Sets the value of the `IP_TTL` option for this socket.
@@ -617,7 +640,7 @@ impl TcpStream {
     /// });
     /// ```
     pub fn set_ttl(&self, ttl: u32) -> Result<()> {
-        Ok(self.stream.stream.set_ttl(ttl)?)
+        Ok(self.stream.stream().set_ttl(ttl)?)
     }
 
     /// Receives data on the socket from the remote address to which it is
@@ -625,7 +648,7 @@ impl TcpStream {
     ///
     /// On success, returns the number of bytes peeked.
     /// Successive calls return the same data. This is accomplished by passing
-    /// MSG_PEEK as a flag to the underlying recv system call.
+    /// MSG_PEEK as a flag to the underlying `recv` system call.
     pub async fn peek(&self, buf: &mut [u8]) -> Result<usize> {
         self.stream.peek(buf).await.map_err(Into::into)
     }
@@ -644,7 +667,7 @@ impl TcpStream {
     /// })
     /// ```
     pub fn peer_addr(&self) -> Result<SocketAddr> {
-        self.stream.stream.peer_addr().map_err(Into::into)
+        self.stream.stream().peer_addr().map_err(Into::into)
     }
 
     /// Returns the socket address of the local half of this TCP connection.
@@ -661,29 +684,25 @@ impl TcpStream {
     /// })
     /// ```
     pub fn local_addr(&self) -> Result<SocketAddr> {
-        self.stream.stream.local_addr().map_err(Into::into)
+        self.stream.stream().local_addr().map_err(Into::into)
     }
 }
 
-impl AsyncBufRead for TcpStream {
+impl<B: Buffered + Unpin> AsyncBufRead for TcpStream<B> {
     fn poll_fill_buf<'a>(
-        mut self: Pin<&'a mut Self>,
+        self: Pin<&'a mut Self>,
         cx: &mut Context<'_>,
     ) -> Poll<io::Result<&'a [u8]>> {
-        let buf_size = self.stream.rx_buf_size;
-        if self.stream.rx_buf.as_ref().is_none() {
-            poll_err!(ready!(self.stream.poll_replenish_buffer(cx, buf_size)));
-        }
         let this = self.project();
-        Poll::Ready(Ok(this.stream.rx_buf.as_ref().unwrap().as_bytes()))
+        this.stream.poll_fill_buf(cx)
     }
 
     fn consume(mut self: Pin<&mut Self>, amt: usize) {
-        Pin::new(&mut self.stream).consume(amt)
+        self.stream.consume(amt);
     }
 }
 
-impl AsyncRead for TcpStream {
+impl<B: RxBuf + Unpin> AsyncRead for TcpStream<B> {
     fn poll_read(
         mut self: Pin<&mut Self>,
         cx: &mut Context<'_>,
@@ -693,7 +712,7 @@ impl AsyncRead for TcpStream {
     }
 }
 
-impl AsyncWrite for TcpStream {
+impl<B: RxBuf + Unpin> AsyncWrite for TcpStream<B> {
     fn poll_write(
         mut self: Pin<&mut Self>,
         cx: &mut Context<'_>,
@@ -766,15 +785,14 @@ mod tests {
             let addr = listener.local_addr().unwrap();
             let coord = Rc::new(Cell::new(0));
 
-            let listener_handle: Task<Result<SocketAddr>> =
-                Task::local(enclose! { (coord) async move {
-                    coord.set(1);
-                    let stream = listener.accept().await?;
-                    stream.peer_addr()
-                }});
+            let listener_handle = crate::spawn_local(enclose! { (coord) async move {
+                coord.set(1);
+                let stream = listener.accept().await?;
+                stream.peer_addr()
+            }});
 
             while coord.get() != 1 {
-                Local::later().await;
+                crate::executor().yield_task_queue_now().await;
             }
             let stream = TcpStream::connect(addr).await.unwrap();
             assert_eq!(listener_handle.await.unwrap(), stream.local_addr().unwrap());
@@ -789,7 +807,7 @@ mod tests {
             let (first_sender, first_receiver) = shared_channel::new_bounded(1);
             let (second_sender, second_receiver) = shared_channel::new_bounded(1);
 
-            let ex1 = LocalExecutorBuilder::new()
+            let ex1 = LocalExecutorBuilder::default()
                 .spawn(move || async move {
                     let receiver = first_receiver.connect().await;
                     let _ = TcpListener::bind(addr).unwrap();
@@ -797,7 +815,7 @@ mod tests {
                 })
                 .unwrap();
 
-            let ex2 = LocalExecutorBuilder::new()
+            let ex2 = LocalExecutorBuilder::default()
                 .spawn(move || async move {
                     let receiver = second_receiver.connect().await;
                     let _ = TcpListener::bind(addr).unwrap();
@@ -824,7 +842,7 @@ mod tests {
         let connected = Arc::new(AtomicUsize::new(0));
 
         let status = connected.clone();
-        let ex1 = LocalExecutorBuilder::new()
+        let ex1 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let sender = sender.connect().await;
                 let addr_sender = addr_sender.connect().await;
@@ -839,7 +857,7 @@ mod tests {
             .unwrap();
 
         let status = connected.clone();
-        let ex2 = LocalExecutorBuilder::new()
+        let ex2 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = receiver.connect().await;
                 let accepted = receiver.recv().await.unwrap();
@@ -848,11 +866,11 @@ mod tests {
             })
             .unwrap();
 
-        let ex3 = LocalExecutorBuilder::new()
+        let ex3 = LocalExecutorBuilder::default()
             .spawn(move || async move {
                 let receiver = addr_receiver.connect().await;
                 let addr = receiver.recv().await.unwrap();
-                TcpStream::connect(addr).await.unwrap()
+                TcpStream::connect(addr).await.unwrap();
             })
             .unwrap();
 
@@ -869,7 +887,7 @@ mod tests {
             let addr = listener.local_addr().unwrap();
             let coord = Rc::new(Cell::new(0));
 
-            let listener_handle: Task<Result<()>> = Task::local(enclose! { (coord) async move {
+            let listener_handle = crate::spawn_local(enclose! { (coord) async move {
                 coord.set(1);
                 listener.incoming().take(4).try_for_each(|addr| {
                     addr.map(|_| ())
@@ -877,11 +895,13 @@ mod tests {
             }});
 
             while coord.get() != 1 {
-                Local::later().await;
+                crate::executor().yield_task_queue_now().await;
             }
             let mut handles = Vec::with_capacity(4);
             for _ in 0..4 {
-                handles.push(Task::local(async move { TcpStream::connect(addr).await }).detach());
+                handles.push(
+                    crate::spawn_local(async move { TcpStream::connect(addr).await }).detach(),
+                );
             }
 
             for handle in handles.drain(..) {
@@ -905,7 +925,7 @@ mod tests {
 
             for _ in 0..128 {
                 handles.push(
-                    Local::local(enclose! { (listener) async move {
+                    crate::spawn_local(enclose! { (listener) async move {
                         let _accept = listener.accept().await.unwrap();
                     }})
                     .detach(),
@@ -919,7 +939,7 @@ mod tests {
             // accept
             for _ in 0..128 {
                 handles.push(
-                    Local::local(async move {
+                    crate::spawn_local(async move {
                         let _stream = TcpStream::connect(addr).await.unwrap();
                     })
                     .detach(),
@@ -939,18 +959,18 @@ mod tests {
             let addr = listener.local_addr().unwrap();
             let coord = Rc::new(Cell::new(0));
 
-            let listener_handle = Task::<Result<u8>>::local(enclose! { (coord) async move {
+            let listener_handle = crate::spawn_local(enclose! { (coord) async move {
                 coord.set(1);
                 let mut stream = listener.accept().await?;
                 let mut byte = [0u8; 1];
                 let read = stream.read(&mut byte).await?;
                 assert_eq!(read, 1);
-                Ok(byte[0])
+                io::Result::Ok(byte[0])
             }})
             .detach();
 
             while coord.get() != 1 {
-                Local::later().await;
+                crate::executor().yield_task_queue_now().await;
             }
             let mut stream = TcpStream::connect(addr).await.unwrap();
 
@@ -967,11 +987,11 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<usize>>::local(async move {
-                let mut stream = listener.accept().await?;
+            let listener_handle = crate::spawn_local(async move {
+                let mut stream = listener.accept().await?.buffered();
                 let mut buf = Vec::new();
                 stream.read_until(10, &mut buf).await?;
-                Ok(buf.len())
+                io::Result::Ok(buf.len())
             })
             .detach();
 
@@ -990,11 +1010,11 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<usize>>::local(async move {
-                let mut stream = listener.accept().await?;
+            let listener_handle = crate::spawn_local(async move {
+                let mut stream = listener.accept().await?.buffered();
                 let mut buf = String::new();
                 stream.read_line(&mut buf).await?;
-                Ok(buf.len())
+                io::Result::Ok(buf.len())
             })
             .detach();
 
@@ -1012,9 +1032,9 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<usize>>::local(async move {
-                let stream = listener.accept().await?;
-                Ok(stream.lines().count().await)
+            let listener_handle = crate::spawn_local(async move {
+                let stream = listener.accept().await?.buffered();
+                io::Result::Ok(stream.lines().count().await)
             })
             .detach();
 
@@ -1033,8 +1053,8 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<()>>::local(async move {
-                let mut stream = listener.accept().await?;
+            let listener_handle = crate::spawn_local(async move {
+                let mut stream = listener.accept().await?.buffered();
                 let buf = stream.fill_buf().await?;
                 // likely both messages were coalesced together
                 assert_eq!(&buf[0..4], b"msg1");
@@ -1044,7 +1064,7 @@ mod tests {
                 stream.consume(4);
                 let buf = stream.fill_buf().await?;
                 assert_eq!(buf.len(), 0);
-                Ok(())
+                io::Result::Ok(())
             })
             .detach();
 
@@ -1065,14 +1085,14 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<()>>::local(async move {
-                let mut stream = listener.accept().await?;
+            let listener_handle = crate::spawn_local(async move {
+                let mut stream = listener.accept().await?.buffered();
                 let buf = stream.fill_buf().await?;
                 assert_eq!(buf.len(), 4);
                 stream.consume(100);
                 let buf = stream.fill_buf().await?;
                 assert_eq!(buf.len(), 0);
-                Ok(())
+                io::Result::Ok(())
             })
             .detach();
 
@@ -1090,8 +1110,8 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<()>>::local(async move {
-                let mut stream = listener.accept().await?;
+            let listener_handle = crate::spawn_local(async move {
+                let mut stream = listener.accept().await?.buffered();
                 let buf = stream.fill_buf().await?;
                 assert_eq!(buf, b"msg1");
                 let buf = stream.fill_buf().await?;
@@ -1099,7 +1119,7 @@ mod tests {
                 stream.consume(4);
                 let buf = stream.fill_buf().await?;
                 assert!(buf.is_empty());
-                Ok(())
+                io::Result::Ok(())
             })
             .detach();
 
@@ -1117,7 +1137,7 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let listener_handle = Task::<Result<usize>>::local(async move {
+            let listener_handle = crate::spawn_local(async move {
                 let mut stream = listener.accept().await?;
                 let mut buf = [0u8; 64];
                 for _ in 0..10 {
@@ -1170,7 +1190,7 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let ltask = Task::<Result<usize>>::local(async move {
+            let ltask = crate::spawn_local(async move {
                 let mut stream = listener.accept().await?;
                 stream
                     .set_read_timeout(Some(Duration::from_secs(1)))
@@ -1184,7 +1204,7 @@ mod tests {
                     }
                 };
                 assert!(now.elapsed().as_secs() >= 1);
-                Ok(0)
+                io::Result::Ok(0)
             });
 
             let _s = TcpStream::connect(addr).await.unwrap();
@@ -1198,7 +1218,7 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let ltask = Task::<Result<usize>>::local(async move {
+            let ltask = crate::spawn_local(async move {
                 let mut stream = listener.accept().await?;
                 poll_fn(|cx| {
                     let mut buf = [0u8; 64];
@@ -1211,7 +1231,7 @@ mod tests {
                     Poll::Ready(())
                 })
                 .await;
-                Ok(0)
+                io::Result::Ok(0)
             });
 
             let _s = TcpStream::connect(addr).await.unwrap();
@@ -1225,7 +1245,7 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let ltask = Task::<Result<usize>>::local(async move {
+            let ltask = crate::spawn_local(async move {
                 let stream = listener.accept().await?;
                 stream
                     .set_write_timeout(Some(Duration::from_nanos(0)))
@@ -1235,7 +1255,7 @@ mod tests {
                     .set_write_timeout(Some(Duration::from_secs(1)))
                     .unwrap();
                 assert_eq!(stream.write_timeout(), Some(Duration::from_secs(1)));
-                Ok(0)
+                io::Result::Ok(0)
             });
 
             let _s = TcpStream::connect(addr).await.unwrap();
@@ -1249,7 +1269,7 @@ mod tests {
             let listener = TcpListener::bind("127.0.0.1:0").unwrap();
             let addr = listener.local_addr().unwrap();
 
-            let peer_addr = Task::local(async move {
+            let peer_addr = crate::spawn_local(async move {
                 let accepted = listener.shared_accept().await.unwrap();
                 let peer_addr = accepted.peer_addr().unwrap();
                 let stream = accepted.bind_to_executor();
