@@ -8,7 +8,7 @@ use nix::sys::socket::{InetAddr, SockAddr};
 use socket2::{Domain, Protocol, Socket, Type};
 use std::{
     io,
-    net::{self, SocketAddr, ToSocketAddrs},
+    net::{self, Ipv4Addr, Ipv6Addr, SocketAddr, ToSocketAddrs},
     os::unix::io::{AsRawFd, FromRawFd, RawFd},
     time::Duration,
 };
@@ -87,7 +87,7 @@ impl UdpSocket {
     /// [`recv`] methods to be used to send data and also applies filters to
     /// only receive data from the specified address.
     ///
-    /// If addr yields multiple addresses, connect will be attempted with each
+    /// If `addr` yields multiple addresses, connect will be attempted with each
     /// of the addresses until the underlying OS function returns no error.
     /// Note that usually, a successful connect call does not specify that
     /// there is a remote server listening on the port, rather, such an
@@ -145,6 +145,257 @@ impl UdpSocket {
         self.socket.rx_buf_size
     }
 
+    /// Gets the value of the `SO_BROADCAST` option for this socket.
+    ///
+    /// For more information about this option, see
+    /// [`UdpSocket::set_broadcast`].
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_broadcast(false).expect("set_broadcast call failed");
+    /// assert_eq!(s.broadcast().unwrap(), false);
+    /// # })
+    /// ```
+    pub fn broadcast(&self) -> Result<bool> {
+        Ok(self.socket.socket.broadcast()?)
+    }
+
+    /// Sets the value of the `SO_BROADCAST` option for this socket.
+    ///
+    /// When enabled, this socket is allowed to send packets to a broadcast
+    /// address.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_broadcast(false).expect("set_broadcast call failed");
+    /// # })
+    /// ```
+    pub fn set_broadcast(&self, broadcast: bool) -> Result<()> {
+        Ok(self.socket.socket.set_broadcast(broadcast)?)
+    }
+
+    /// Executes an operation of the `IP_ADD_MEMBERSHIP` type.
+    ///
+    /// This function specifies a new multicast group for this socket to join.
+    /// The address must be a valid multicast address, and `interface` is the
+    /// address of the local interface with which the system should join the
+    /// multicast group. If it's equal to `INADDR_ANY` then an appropriate
+    /// interface is chosen by the system.
+    pub fn join_multicast_v4(&self, multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> Result<()> {
+        Ok(self.socket.socket.join_multicast_v4(multiaddr, interface)?)
+    }
+
+    /// Executes an operation of the `IPV6_ADD_MEMBERSHIP` type.
+    ///
+    /// This function specifies a new multicast group for this socket to join.
+    /// The address must be a valid multicast address, and `interface` is the
+    /// index of the interface to join/leave (or 0 to indicate any interface).
+    pub fn join_multicast_v6(&self, multiaddr: &Ipv6Addr, interface: u32) -> Result<()> {
+        Ok(self.socket.socket.join_multicast_v6(multiaddr, interface)?)
+    }
+
+    /// Executes an operation of the `IP_DROP_MEMBERSHIP` type.
+    ///
+    /// For more information about this option, see
+    /// [`UdpSocket::join_multicast_v4`].
+    pub fn leave_multicast_v4(&self, multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> Result<()> {
+        Ok(self
+            .socket
+            .socket
+            .leave_multicast_v4(multiaddr, interface)?)
+    }
+
+    /// Executes an operation of the `IPV6_DROP_MEMBERSHIP` type.
+    ///
+    /// For more information about this option, see
+    /// [`UdpSocket::join_multicast_v6`].
+    pub fn leave_multicast_v6(&self, multiaddr: &Ipv6Addr, interface: u32) -> Result<()> {
+        Ok(self
+            .socket
+            .socket
+            .leave_multicast_v6(multiaddr, interface)?)
+    }
+
+    /// Gets the value of the `IP_MULTICAST_LOOP` option for this socket.
+    ///
+    /// For more information about this option, see
+    /// [`UdpSocket::set_multicast_loop_v4`].
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_multicast_loop_v4(false)
+    ///     .expect("set_multicast_loop_v4 call failed");
+    /// assert_eq!(s.multicast_loop_v4().unwrap(), false);
+    /// # })
+    /// ```
+    pub fn multicast_loop_v4(&self) -> Result<bool> {
+        Ok(self.socket.socket.multicast_loop_v4()?)
+    }
+
+    /// Sets the value of the `IP_MULTICAST_LOOP` option for this socket.
+    ///
+    /// If enabled, multicast packets will be looped back to the local socket.
+    /// Note that this may not have any effect on IPv6 sockets.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_multicast_loop_v4(false)
+    ///     .expect("set_multicast_loop_v4 call failed");
+    /// # })
+    /// ```
+    pub fn set_multicast_loop_v4(&self, multicast_loop_v4: bool) -> Result<()> {
+        Ok(self
+            .socket
+            .socket
+            .set_multicast_loop_v4(multicast_loop_v4)?)
+    }
+
+    /// Gets the value of the `IPV6_MULTICAST_LOOP` option for this socket.
+    ///
+    /// For more information about this option, see
+    /// [`UdpSocket::set_multicast_loop_v6`].
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_multicast_loop_v6(false)
+    ///     .expect("set_multicast_loop_v6 call failed");
+    /// assert_eq!(s.multicast_loop_v6().unwrap(), false);
+    /// # })
+    /// ```
+    pub fn multicast_loop_v6(&self) -> Result<bool> {
+        Ok(self.socket.socket.multicast_loop_v6()?)
+    }
+
+    /// Sets the value of the `IPV6_MULTICAST_LOOP` option for this socket.
+    ///
+    /// Controls whether this socket sees the multicast packets it sends itself.
+    /// Note that this may not have any affect on IPv4 sockets.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_multicast_loop_v6(false)
+    ///     .expect("set_multicast_loop_v6 call failed");
+    /// # })
+    /// ```
+    pub fn set_multicast_loop_v6(&self, multicast_loop_v6: bool) -> Result<()> {
+        Ok(self
+            .socket
+            .socket
+            .set_multicast_loop_v6(multicast_loop_v6)?)
+    }
+
+    /// Gets the value of the `IP_MULTICAST_TTL` option for this socket.
+    ///
+    /// For more information about this option, see
+    /// [`UdpSocket::set_multicast_ttl_v4`].
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_multicast_ttl_v4(42)
+    ///     .expect("set_multicast_ttl_v4 call failed");
+    /// assert_eq!(s.multicast_ttl_v4().unwrap(), 42);
+    /// # })
+    /// ```
+    pub fn multicast_ttl_v4(&self) -> Result<u32> {
+        Ok(self.socket.socket.multicast_ttl_v4()?)
+    }
+
+    /// Sets the value of the `IP_MULTICAST_TTL` option for this socket.
+    ///
+    /// Indicates the time-to-live value of outgoing multicast packets for
+    /// this socket. The default value is 1 which means that multicast packets
+    /// don't leave the local network unless explicitly requested.
+    ///
+    /// Note that this may not have any effect on IPv6 sockets.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_multicast_ttl_v4(42)
+    ///     .expect("set_multicast_ttl_v4 call failed");
+    /// # })
+    /// ```
+    pub fn set_multicast_ttl_v4(&self, multicast_ttl_v4: u32) -> Result<()> {
+        Ok(self.socket.socket.set_multicast_ttl_v4(multicast_ttl_v4)?)
+    }
+
+    /// Gets the value of the `IP_TTL` option for this socket.
+    ///
+    /// For more information about this option, see [`UdpSocket::set_ttl`].
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_ttl(42).expect("set_ttl call failed");
+    /// assert_eq!(s.ttl().unwrap(), 42);
+    /// # })
+    /// ```
+    pub fn ttl(&self) -> Result<u32> {
+        Ok(self.socket.socket.ttl()?)
+    }
+
+    /// Sets the value for the `IP_TTL` option on this socket.
+    ///
+    /// This value sets the time-to-live field that is used in every packet sent
+    /// from this socket.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// # use glommio::{net::UdpSocket, LocalExecutor};
+    /// # let ex = LocalExecutor::default();
+    /// # ex.run(async move {
+    /// let s = UdpSocket::bind("127.0.0.1:10000").unwrap();
+    /// s.set_ttl(42).expect("set_ttl call failed");
+    /// # })
+    /// ```
+    pub fn set_ttl(&self, ttl: u32) -> Result<()> {
+        Ok(self.socket.socket.set_ttl(ttl)?)
+    }
+
     /// Sets the read timeout to the timeout specified.
     ///
     /// If the value specified is [`None`], then read calls will block
@@ -342,8 +593,8 @@ impl UdpSocket {
     /// Sends data on the socket to the given address. On success, returns the
     /// number of bytes written. Address type can be any implementor of
     /// [`ToSocketAddrs`] trait. See its documentation for concrete examples.
-    /// It is possible for addr to yield multiple addresses, but send_to will
-    /// only send data to the first address yielded by addr.
+    /// It is possible for `addr` to yield multiple addresses, but send_to will
+    /// only send data to the first address yielded by `addr`.
     ///
     /// # Examples
     ///
@@ -409,7 +660,7 @@ impl UdpSocket {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{timer::Timer, Local, LocalExecutorBuilder};
+    use crate::{timer::Timer, LocalExecutorBuilder};
     use nix::sys::socket::MsgFlags;
     use std::time::Duration;
 
@@ -458,7 +709,7 @@ mod tests {
             let addr_picker = UdpSocket::bind("127.0.0.1:0").unwrap();
             let addr = addr_picker.local_addr().unwrap();
 
-            let ex1 = LocalExecutorBuilder::new()
+            let ex1 = LocalExecutorBuilder::default()
                 .spawn(move || async move {
                     let socket = UdpSocket::bind(addr).unwrap();
                     let mut buf = [0u8; 1];
@@ -470,7 +721,7 @@ mod tests {
                 })
                 .unwrap();
 
-            let ex2 = LocalExecutorBuilder::new()
+            let ex2 = LocalExecutorBuilder::default()
                 .spawn(move || async move {
                     let socket = UdpSocket::bind(addr).unwrap();
                     let mut buf = [0u8; 1];
@@ -558,7 +809,7 @@ mod tests {
         test_executor!(async move {
             let (receiver, sender) = connected_pair!();
 
-            let recv_handle = Local::local(async move {
+            let recv_handle = crate::spawn_local(async move {
                 let mut buf = [0u8; 10];
                 // try to receive 10 bytes, but will assert that none comes back.
                 let sz = receiver.recv(&mut buf).await.unwrap();
@@ -578,7 +829,7 @@ mod tests {
         test_executor!(async move {
             let (receiver, sender) = connected_pair!();
 
-            let receiver_handle = Task::local(async move {
+            let receiver_handle = crate::spawn_local(async move {
                 for _ in 0..10 {
                     let mut buf = [0u8; 40];
                     let sz = receiver.peek(&mut buf).await.unwrap();
@@ -625,7 +876,7 @@ mod tests {
 
             let addr = receiver.local_addr().unwrap();
 
-            let receive_handle = Task::local(async move {
+            let receive_handle = crate::spawn_local(async move {
                 let mut buf = [0u8; 1];
                 for _ in 0..10 {
                     let (sz, _) = receiver
@@ -683,7 +934,7 @@ mod tests {
 
             let addr = receiver.local_addr().unwrap();
 
-            let receive_handle = Task::local(async move {
+            let receive_handle = crate::spawn_local(async move {
                 let mut buf = [0u8; 1];
                 let (sz, from) = receiver
                     .socket
@@ -746,4 +997,77 @@ mod tests {
             assert_eq!(buf[0], 65u8);
         });
     }
+
+    #[test]
+    fn broadcast() {
+        test_executor!(async move {
+            let s = UdpSocket::bind("127.0.0.1:0").unwrap();
+            s.set_broadcast(false).expect("set_broadcast call failed");
+            assert!(!s.broadcast().unwrap());
+        });
+    }
+
+    #[test]
+    fn multicast_v4() {
+        test_executor!(async move {
+            let multicast_addr = Ipv4Addr::new(239, 0, 0, 0);
+            assert!(multicast_addr.is_multicast());
+            let interface_addr = Ipv4Addr::new(0, 0, 0, 0);
+            let s = UdpSocket::bind("127.0.0.1:0").unwrap();
+            s.join_multicast_v4(&multicast_addr, &interface_addr)
+                .unwrap();
+            s.leave_multicast_v4(&multicast_addr, &interface_addr)
+                .unwrap();
+        });
+    }
+
+    #[test]
+    fn multicast_v6() {
+        test_executor!(async move {
+            let multicast_addr = Ipv6Addr::new(0xff00, 0, 0, 0, 0, 0, 0, 0);
+            assert!(multicast_addr.is_multicast());
+            let s = UdpSocket::bind("::1:0").unwrap();
+            s.join_multicast_v6(&multicast_addr, 0).unwrap();
+            s.leave_multicast_v6(&multicast_addr, 0).unwrap();
+        });
+    }
+
+    #[test]
+    fn set_multicast_loop_v4() {
+        test_executor!(async move {
+            let s = UdpSocket::bind("127.0.0.1:0").unwrap();
+            s.set_multicast_loop_v4(false)
+                .expect("set_multicast_loop_v4 call failed");
+            assert!(!s.multicast_loop_v4().unwrap());
+        });
+    }
+
+    #[test]
+    fn set_multicast_loop_v6() {
+        test_executor!(async move {
+            let s = UdpSocket::bind("::1:0").unwrap();
+            s.set_multicast_loop_v6(false)
+                .expect("set_multicast_loop_v6 call failed");
+            assert!(!s.multicast_loop_v6().unwrap());
+        });
+    }
+
+    #[test]
+    fn set_multicast_ttl_v4() {
+        test_executor!(async move {
+            let s = UdpSocket::bind("127.0.0.1:0").unwrap();
+            s.set_multicast_ttl_v4(42)
+                .expect("set_multicast_ttl_v4 call failed");
+            assert_eq!(s.multicast_ttl_v4().unwrap(), 42);
+        });
+    }
+
+    #[test]
+    fn set_ttl() {
+        test_executor!(async move {
+            let s = UdpSocket::bind("127.0.0.1:0").unwrap();
+            s.set_ttl(42).expect("set_ttl call failed");
+            assert_eq!(s.ttl().unwrap(), 42);
+        });
+    }
 }
