@@ -4,11 +4,13 @@
 // This product includes software developed at Datadog (https://www.datadoghq.com/). Copyright 2020 Datadog, Inc.
 //
 use super::{datagram::GlommioDatagram, stream::GlommioStream};
-use crate::{parking::Reactor, Local};
+use crate::{
+    net::stream::{Buffered, NonBuffered, Preallocated, RxBuf},
+    reactor::Reactor,
+};
 use futures_lite::{
     future::poll_fn,
     io::{AsyncBufRead, AsyncRead, AsyncWrite},
-    ready,
     stream::{self, Stream},
 };
 use nix::sys::socket::{SockAddr, UnixAddr};
@@ -97,7 +99,7 @@ impl UnixListener {
         let listener = sk.into_unix_listener();
 
         Ok(UnixListener {
-            reactor: Rc::downgrade(&Local::get_reactor()),
+            reactor: Rc::downgrade(&crate::executor().reactor()),
             listener,
         })
     }
@@ -247,7 +249,7 @@ impl AcceptedUnixStream {
     ///     let accepted = listener.shared_accept().await.unwrap();
     ///     sender.try_send(accepted).unwrap();
     ///
-    ///     let ex1 = LocalExecutorBuilder::new()
+    ///     let ex1 = LocalExecutorBuilder::default()
     ///         .spawn(move || async move {
     ///             let receiver = receiver.connect().await;
     ///             let accepted = receiver.recv().await.unwrap();
@@ -272,8 +274,16 @@ pin_project! {
     /// [`AsyncRead`]: https://docs.rs/futures-io/0.3.8/futures_io/trait.AsyncRead.html
     /// [`AsyncBufRead`]: https://docs.rs/futures-io/0.3.8/futures_io/trait.AsyncBufRead.html
     /// [`AsyncWrite`]: https://docs.rs/futures-io/0.3.8/futures_io/trait.AsyncWrite.html
-    pub struct UnixStream {
-        stream: GlommioStream<net::UnixStream>
+    pub struct UnixStream<B: RxBuf = NonBuffered> {
+        stream: GlommioStream<net::UnixStream, B>
+    }
+}
+
+impl FromRawFd for UnixStream {
+    unsafe fn from_raw_fd(fd: RawFd) -> Self {
+        UnixStream {
+            stream: GlommioStream::from_raw_fd(fd as _),
+        }
     }
 }
 
@@ -316,7 +326,7 @@ impl UnixStream {
     /// })
     /// ```
     pub async fn connect<A: AsRef<Path>>(addr: A) -> Result<UnixStream> {
-        let reactor = Local::get_reactor();
+        let reactor = crate::executor().reactor();
 
         let socket = Socket::new(Domain::unix(), Type::stream(), None)?;
         let addr = SockAddr::new_unix(addr.as_ref())
@@ -329,6 +339,20 @@ impl UnixStream {
         })
     }
 
+    /// Creates a buffered Unix connection with default receive buffer.
+    pub fn buffered(self) -> UnixStream<Preallocated> {
+        self.buffered_with(Preallocated::default())
+    }
+
+    /// Creates a buffered Unix connection with custom receive buffer.
+    pub fn buffered_with<B: Buffered>(self, buf: B) -> UnixStream<B> {
+        UnixStream {
+            stream: self.stream.buffered_with(buf),
+        }
+    }
+}
+
+impl<B: RxBuf> UnixStream<B> {
     /// Shuts down the read, write, or both halves of this connection.
     pub async fn shutdown(&self, how: Shutdown) -> Result<()> {
         poll_fn(|cx| self.stream.poll_shutdown(cx, how))
@@ -336,16 +360,6 @@ impl UnixStream {
             .map_err(Into::into)
     }
 
-    /// Sets the buffer size used on the receive path
-    pub fn set_buffer_size(&mut self, buffer_size: usize) {
-        self.stream.rx_buf_size = buffer_size;
-    }
-
-    /// gets the buffer size used
-    pub fn buffer_size(&mut self) -> usize {
-        self.stream.rx_buf_size
-    }
-
     /// Receives data on the socket from the remote address to which it is
     /// connected, without removing that data from the queue.
     ///
@@ -370,7 +384,7 @@ impl UnixStream {
     /// })
     /// ```
     pub fn peer_addr(&self) -> Result<SocketAddr> {
-        self.stream.stream.peer_addr().map_err(Into::into)
+        self.stream.stream().peer_addr().map_err(Into::into)
     }
 
     /// Returns the socket address of the local half of this Unix connection.
@@ -387,29 +401,25 @@ impl UnixStream {
     /// })
     /// ```
     pub fn local_addr(&self) -> Result<SocketAddr> {
-        self.stream.stream.local_addr().map_err(Into::into)
+        self.stream.stream().local_addr().map_err(Into::into)
     }
 }
 
-impl AsyncBufRead for UnixStream {
+impl<B: Buffered + Unpin> AsyncBufRead for UnixStream<B> {
     fn poll_fill_buf<'a>(
-        mut self: Pin<&'a mut Self>,
+        self: Pin<&'a mut Self>,
         cx: &mut Context<'_>,
     ) -> Poll<io::Result<&'a [u8]>> {
-        let buf_size = self.stream.rx_buf_size;
-        if self.stream.rx_buf.as_ref().is_none() {
-            poll_err!(ready!(self.stream.poll_replenish_buffer(cx, buf_size)));
-        }
         let this = self.project();
-        Poll::Ready(Ok(this.stream.rx_buf.as_ref().unwrap().as_bytes()))
+        this.stream.poll_fill_buf(cx)
     }
 
     fn consume(mut self: Pin<&mut Self>, amt: usize) {
-        Pin::new(&mut self.stream).consume(amt)
+        self.stream.consume(amt);
     }
 }
 
-impl AsyncRead for UnixStream {
+impl<B: RxBuf + Unpin> AsyncRead for UnixStream<B> {
     fn poll_read(
         mut self: Pin<&mut Self>,
         cx: &mut Context<'_>,
@@ -419,7 +429,7 @@ impl AsyncRead for UnixStream {
     }
 }
 
-impl AsyncWrite for UnixStream {
+impl<B: RxBuf + Unpin> AsyncWrite for UnixStream<B> {
     fn poll_write(
         mut self: Pin<&mut Self>,
         cx: &mut Context<'_>,
@@ -502,7 +512,7 @@ impl UnixDatagram {
     /// the [`send`] and [`recv`] methods to be used to send data and also
     /// applies filters to only receive data from the specified address.
     ///
-    /// If addr yields multiple addresses, connect will be attempted with each
+    /// If `addr` yields multiple addresses, connect will be attempted with each
     /// of the addresses until the underlying OS function returns no error.
     /// Note that usually, a successful connect call does not specify that
     /// there is a remote server listening on the port, rather, such an
@@ -577,7 +587,7 @@ impl UnixDatagram {
         self.socket.peek(buf).await.map_err(Into::into)
     }
 
-    ///Receives a single datagram message on the socket, without removing it
+    /// Receives a single datagram message on the socket, without removing it
     /// from the queue. On success, returns the number of bytes read and the
     /// origin.
     ///
@@ -744,14 +754,14 @@ mod tests {
         let addr = addr.as_pathname().unwrap();
         let coord = Rc::new(Cell::new(0));
 
-        let listener_handle = Task::local(enclose! { (coord) async move {
+        let listener_handle = crate::spawn_local(enclose! { (coord) async move {
             coord.set(1);
             listener.accept().await.unwrap();
         }})
         .detach();
 
         while coord.get() != 1 {
-            Local::later().await;
+            crate::executor().yield_task_queue_now().await;
         }
         UnixStream::connect(&addr).await.unwrap();
         listener_handle.await.unwrap();
@@ -773,11 +783,11 @@ mod tests {
 
         let listener = UnixListener::bind(&file).unwrap();
 
-        let listener_handle = Task::<io::Result<usize>>::local(async move {
-            let mut stream = listener.accept().await?;
+        let listener_handle = crate::spawn_local(async move {
+            let mut stream = listener.accept().await?.buffered();
             let mut buf = Vec::new();
             stream.read_until(10, &mut buf).await?;
-            Ok(buf.len())
+            io::Result::Ok(buf.len())
         })
         .detach();
 
