@@ -36,9 +36,9 @@ struct WaiterNode {
     units: u64,
     waker: RefCell<Option<Waker>>,
 
-    //waiter node can not be Unpin so its pointer could be used inside of intrusive
-    //collection, it also can not outlive the container which is guaranteed by the
-    //Waiter lifetime bound to the Semaphore which is container of all Waiters.
+    // Waiter node can not be `Unpin` so its pointer could be used inside intrusive
+    // collections, it also can not outlive the container which is guaranteed by the
+    // Waiter lifetime bound to the Semaphore which is container of all Waiters.
     _p: PhantomPinned,
 }
 
@@ -91,7 +91,7 @@ unsafe impl Adapter for WaiterAdapter {
         }
 
         let ptr = (value as *const u8).add(offset_of!(WaiterNode, link));
-        //we call unchecked method because of safety check above
+        // We call unchecked method because of safety check above
         core::ptr::NonNull::new_unchecked(ptr as *mut _)
     }
 
@@ -150,7 +150,7 @@ impl<'a> Waiter<'a> {
         }
 
         sem_state.waiters_list.push_back(unsafe {
-            //it is safe to use unchecked call here because we convert passed in reference
+            // It is safe to use unchecked call here because we convert passed in reference
             // which can not be null
             NonNull::new_unchecked(Pin::into_inner_unchecked(waiter_node) as *mut _)
         });
@@ -160,8 +160,8 @@ impl<'a> Waiter<'a> {
 impl<'a> Drop for Waiter<'a> {
     fn drop(&mut self) {
         if self.node.link.is_linked() {
-            //if node is linked that is for sure is pinned so that is safe
-            //to make it pinned directly
+            // If node is linked that is for sure is pinned so that is safe
+            // to make it pinned directly
             let waiter_node = unsafe { Pin::new_unchecked(&mut self.node) };
             Self::remove_from_waiting_queue(waiter_node, &mut self.semaphore.state.borrow_mut())
         }
@@ -387,7 +387,7 @@ impl Semaphore {
     /// Similar to [`acquire_static_permit`], except that it requires the permit
     /// never to be passed to contexts that require a static lifetime. As
     /// this is cheaper than [`acquire_static_permit`], it is useful in
-    /// situations where the permit tracks an a asynchronous operation whose
+    /// situations where the permit tracks an asynchronous operation whose
     /// lifetime is simple and well-defined.
     ///
     /// # Examples
@@ -427,7 +427,7 @@ impl Semaphore {
     /// # Examples
     ///
     /// ```
-    /// use glommio::{sync::Semaphore, timer::sleep, Local, LocalExecutor};
+    /// use glommio::{sync::Semaphore, timer::sleep, LocalExecutor};
     /// use std::{rc::Rc, time::Duration};
     ///
     /// let sem = Rc::new(Semaphore::new(1));
@@ -436,7 +436,7 @@ impl Semaphore {
     /// ex.run(async move {
     ///     {
     ///         let permit = sem.acquire_static_permit(1).await.unwrap();
-    ///         Local::local(async move {
+    ///         glommio::spawn_local(async move {
     ///             let _guard = permit;
     ///             sleep(Duration::from_secs(1)).await;
     ///         })
@@ -535,7 +535,7 @@ impl Semaphore {
     /// reducing the number of available units by the given amount.
     ///
     /// The [`Permit`] is bound to the lifetime of the current reference of this
-    /// semaphore. If you are need it to live longer, consider using
+    /// semaphore. If you need it to live longer, consider using
     /// [`try_acquire_static_permit`].
     ///
     /// If insufficient units are available then this method will return
@@ -581,11 +581,11 @@ impl Semaphore {
     }
 
     /// Acquires the given number of units, if they are available, and
-    /// returns immediately, with the RAAI guard,
+    /// returns immediately, with the RAII guard,
     /// reducing the number of available units by the given amount.
     ///
     /// This function returns a [`StaticPermit`]
-    /// and is suitable for `'static` contexts. If your lifetimes are simple
+    /// and is suitable for `'static` contexts. If your lifetimes are simple,
     /// and you don't need the permit to outlive your context, consider using
     /// [`try_acquire_permit`] instead.
     ///
@@ -634,8 +634,9 @@ impl Semaphore {
 
     /// Signals the semaphore to release the specified amount of units.
     ///
-    /// This needs to be paired with a call to acquire(). You should not
-    /// call this if the units were acquired with acquire_permit().
+    /// This needs to be paired with a call to [`Semaphore::acquire()`]. You
+    /// should not call this if the units were acquired with
+    /// [`Semaphore::acquire_permit()`].
     ///
     /// # Examples
     ///
@@ -657,7 +658,8 @@ impl Semaphore {
 
     /// Closes the semaphore
     ///
-    /// All existing waiters will return Err(), and no new waiters are allowed.
+    /// All existing waiters will return `Err()`, and no new waiters are
+    /// allowed.
     ///
     /// # Examples
     ///
@@ -693,7 +695,6 @@ mod test {
     use crate::{
         enclose,
         timer::{sleep, Timer},
-        Local,
         LocalExecutor,
     };
 
@@ -719,20 +720,20 @@ mod test {
             let sem = Rc::new(Semaphore::new(0));
             let exec = Rc::new(Cell::new(0));
 
-            let t1 = Local::local(enclose! { (sem, exec) async move {
+            let t1 = crate::spawn_local(enclose! { (sem, exec) async move {
                 exec.set(exec.get() + 1);
                 let _g = sem.acquire_permit(1).await.unwrap();
             }});
-            let t2 = Task::local(enclose! { (sem, exec) async move {
+            let t2 = crate::spawn_local(enclose! { (sem, exec) async move {
                 exec.set(exec.get() + 1);
                 let _g = sem.acquire_permit(1).await.unwrap();
             }});
 
             // For this last one, use the static version. Move around and sleep a bit first
-            let t3 = Local::local(enclose! { (sem, exec) async move {
+            let t3 = crate::spawn_local(enclose! { (sem, exec) async move {
                 exec.set(exec.get() + 1);
                 let g = sem.acquire_static_permit(1).await.unwrap();
-                Local::local(async move {
+                crate::spawn_local(async move {
                     let _g = g;
                     sleep(Duration::from_secs(1)).await;
                 }).await
@@ -740,7 +741,7 @@ mod test {
 
             // Wait for all permits to try and acquire, then unleash the gates.
             while exec.get() != 3 {
-                Local::later().await;
+                crate::executor().yield_task_queue_now().await;
             }
             sem.signal(1);
 
@@ -934,7 +935,7 @@ mod test {
         let semaphore_c = semaphore.clone();
 
         ex.run(async move {
-            Local::local(async move {
+            crate::spawn_local(async move {
                 for _ in 0..100 {
                     Timer::new(Duration::from_micros(100)).await;
                 }
@@ -970,16 +971,16 @@ mod test {
         let state_c3 = state.clone();
 
         ex.run(async move {
-            let t1 = Local::local(async move {
+            let t1 = crate::spawn_local(async move {
                 *state_c1.borrow_mut() = 1;
                 let _g = semaphore_c1.acquire_permit(1).await.unwrap();
                 assert_eq!(*state_c1.borrow(), 3);
                 *state_c1.borrow_mut() = 4;
             });
 
-            let t2 = Local::local(async move {
+            let t2 = crate::spawn_local(async move {
                 while *state_c2.borrow() != 1 {
-                    Local::later().await;
+                    crate::executor().yield_task_queue_now().await;
                 }
 
                 *state_c2.borrow_mut() = 2;
@@ -988,9 +989,9 @@ mod test {
                 *state_c2.borrow_mut() = 5;
             });
 
-            let t3 = Local::local(async move {
+            let t3 = crate::spawn_local(async move {
                 while *state_c3.borrow() != 2 {
-                    Local::later().await;
+                    crate::executor().yield_task_queue_now().await;
                 }
 
                 *state_c3.borrow_mut() = 3;
@@ -998,9 +999,9 @@ mod test {
                 assert_eq!(*state_c3.borrow(), 5);
             });
 
-            Local::local(async move {
+            crate::spawn_local(async move {
                 while *state.borrow() != 3 {
-                    Local::later().await;
+                    crate::executor().yield_task_queue_now().await;
                 }
 
                 semaphore.signal(1);
