@@ -6,7 +6,7 @@
 //
 
 use std::{
-    collections::{HashMap, HashSet},
+    collections::{BTreeMap, HashMap, HashSet},
     io::{self, ErrorKind},
     path::Path,
 };
@@ -14,7 +14,7 @@ use std::{
 use super::sysfs::ListIterator;
 
 /// A description of the CPU's location in the machine topology.
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, Eq, Hash, PartialEq)]
 pub struct CpuLocation {
     /// Holds the CPU id.  This is the most granular field and will distinguish
     /// among [`hyper-threads`].
@@ -93,6 +93,40 @@ pub fn get_machine_topology_unsorted() -> io::Result<Vec<CpuLocation>> {
         }
     }
 
+    // Assign a virtual core id to each CPU. The basic strategy is to sort CPUs
+    // by their (NUMA node id, core id) and assign virtual core id in this order.
+    // Note we need to ensure that the CPUs on the same core will have the same core
+    // id.
+
+    // Using BTree over HashMap for 2 reasons:
+    // 1. to keep mapping consitent between different invocations.
+    // 2. to assign smaller virtual core ids to smaller numa node id.
+    // numa_node -> (core_id -> [cpu_id])
+    let mut node_to_core_to_cpus: BTreeMap<usize, BTreeMap<usize, Vec<usize>>> = BTreeMap::new();
+    for l in &cpu_locations {
+        node_to_core_to_cpus
+            .entry(l.numa_node)
+            .or_insert_with(BTreeMap::new)
+            .entry(l.core)
+            .or_insert_with(Vec::new)
+            .push(l.cpu)
+    }
+
+    let mut cpu_to_vcore = HashMap::new();
+    for (vcore, cpu_on_same_core) in node_to_core_to_cpus
+        .values()
+        .flat_map(BTreeMap::values)
+        .enumerate()
+    {
+        for &cpu in cpu_on_same_core {
+            cpu_to_vcore.insert(cpu, vcore);
+        }
+    }
+
+    for cpu_location in &mut cpu_locations {
+        cpu_location.core = *cpu_to_vcore.get(&cpu_location.cpu).unwrap();
+    }
+
     Ok(cpu_locations)
 }
 
@@ -126,8 +160,8 @@ pub(crate) mod test_helpers {
     use super::{CpuLocation, HashMap};
 
     pub fn check_topolgy(mut topology: Vec<CpuLocation>) {
-        // check that we don't have a system where any hardware component has an id that
-        // is not unique system wide (e.g. both numa node 0 and 1 have a core
+        // Check that we don't have a system where any hardware component has an id that
+        // is not unique system-wide (e.g. both numa node 0 and 1 have a core
         // with id 0); this precondition is assumed throughout
         topology.sort_by_key(|l| (l.numa_node, l.package, l.core, l.cpu));
 
@@ -142,12 +176,10 @@ pub(crate) mod test_helpers {
             cpu_to_core
                 .entry(cpu.cpu)
                 .and_modify(|e| {
-                    assert!(
-                        *e == cpu.core,
+                    assert_eq!(
+                        *e, cpu.core,
                         "cpu {} in cores {} and {}",
-                        cpu.cpu,
-                        cpu.core,
-                        *e
+                        cpu.cpu, cpu.core, *e
                     )
                 })
                 .or_insert(cpu.core);
@@ -155,12 +187,10 @@ pub(crate) mod test_helpers {
             core_to_pkg
                 .entry(cpu.core)
                 .and_modify(|e| {
-                    assert!(
-                        *e == cpu.package,
+                    assert_eq!(
+                        *e, cpu.package,
                         "core {} in packages {} and {}",
-                        cpu.core,
-                        cpu.package,
-                        *e
+                        cpu.core, cpu.package, *e
                     )
                 })
                 .or_insert(cpu.package);
@@ -168,12 +198,10 @@ pub(crate) mod test_helpers {
             core_to_numa
                 .entry(cpu.core)
                 .and_modify(|e| {
-                    assert!(
-                        *e == cpu.numa_node,
+                    assert_eq!(
+                        *e, cpu.numa_node,
                         "core {} in numa_nodes {} and {}",
-                        cpu.core,
-                        cpu.numa_node,
-                        *e
+                        cpu.core, cpu.numa_node, *e
                     )
                 })
                 .or_insert(cpu.numa_node);
