@@ -35,6 +35,7 @@ impl FromStr for StorageCache {
 }
 
 #[derive(Debug)]
+#[allow(dead_code)]
 pub(crate) struct BlockDevice {
     memory_device: bool,
     rotational: bool,
@@ -42,6 +43,8 @@ pub(crate) struct BlockDevice {
     optimal_io_size: usize,
     logical_block_size: usize,
     physical_block_size: usize,
+    max_sectors_size: usize,
+    max_segment_size: usize,
     cache: StorageCache,
     subcomponents: Vec<PathBuf>,
 }
@@ -76,6 +79,8 @@ impl BlockDevice {
             optimal_io_size: 128 << 10,
             logical_block_size: 512,
             physical_block_size: 512,
+            max_sectors_size: 128 << 10,
+            max_segment_size: (u32::MAX - 1) as usize,
             cache: StorageCache::WriteBack,
             subcomponents: Vec::new(),
         }
@@ -100,6 +105,8 @@ impl BlockDevice {
         let optimal_io_size = read_int(&queue.join("optimal_io_size")) as _;
         let logical_block_size = read_int(&queue.join("logical_block_size")) as _;
         let physical_block_size = read_int(&queue.join("physical_block_size")) as _;
+        let max_sectors_kb = read_int(&queue.join("max_sectors_kb")) as usize;
+        let max_segment_size = read_int(&queue.join("max_segment_size")) as usize;
 
         let cache_data = read_to_string(&queue.join("write_cache")).unwrap();
         let cache = cache_data.parse::<StorageCache>().unwrap();
@@ -115,6 +122,8 @@ impl BlockDevice {
             optimal_io_size,
             logical_block_size,
             physical_block_size,
+            max_sectors_size: max_sectors_kb << 10,
+            max_segment_size,
             cache,
             subcomponents,
         }
@@ -144,6 +153,14 @@ impl BlockDevice {
         block_property!(DEV_MAP, physical_block_size, major, minor)
     }
 
+    pub(crate) fn max_sectors_size(major: usize, minor: usize) -> usize {
+        block_property!(DEV_MAP, max_sectors_size, major, minor)
+    }
+
+    pub(crate) fn max_segment_size(major: usize, minor: usize) -> usize {
+        block_property!(DEV_MAP, max_segment_size, major, minor)
+    }
+
     pub(crate) fn is_md(major: usize, minor: usize) -> bool {
         !block_property!(DEV_MAP, subcomponents, major, minor).is_empty()
     }
@@ -210,7 +227,7 @@ impl ListIterator {
             .get(beg..)
             .expect("invalid range: this is a bug")
             .find(|c: char| !c.is_ascii_digit())
-            .unwrap_or_else(|| self.list_str.len() - beg);
+            .unwrap_or(self.list_str.len() - beg);
 
         self.list_str
             .get(beg..self.idx)
@@ -240,7 +257,7 @@ impl ListIterator {
             .get(self.idx..idx_max)
             .expect("invalid range: this is a bug")
             .find(|c| c != ',' && !is_space(c))
-            .unwrap_or_else(|| idx_max - self.idx);
+            .unwrap_or(idx_max - self.idx);
     }
 
     fn skip_char(&mut self, chr: char) -> bool {
@@ -306,7 +323,7 @@ impl ListIterator {
         io::Error::new(io::ErrorKind::InvalidData, msg)
     }
 
-    // returns the first error if any elements were errors, otherwise returns the
+    // Returns the first error if any elements were errors, otherwise returns the
     // collection of unwrapped elements
     #[cfg(test)]
     fn collect_ok<C>(self) -> io::Result<C>
@@ -496,7 +513,7 @@ pub(super) mod test_helpers {
         }
 
         fn set_on_deck(&mut self) -> Option<u64> {
-            // count unused bits (these are all equal to 0)
+            // Count unused bits (these are all equal to 0)
             self.bit_counter += self.n_bits_on_deck;
 
             self.s_end = 1 + self
@@ -509,7 +526,7 @@ pub(super) mod test_helpers {
                 .rfind(|c: char| !c.is_ascii_hexdigit())
                 .map_or(0, |ii| ii + 1);
 
-            // each hexadecimal char represents 4 bits and we only have space for 64 bits
+            // Each hexadecimal char represents 4 bits and we only have space for 64 bits
             if self.s_end - self.s_beg > 16 {
                 self.s_beg = self.s_end - 16;
             }
