@@ -20,13 +20,16 @@ use crate::{
 ///
 /// This header is stored right at the beginning of every heap-allocated task.
 pub(crate) struct Header {
-    /// ID of the executor to which task belongs to or in another words by which
+    /// ID of the executor to which task belongs to or in other words by which
     /// task was spawned by
     pub(crate) notifier: Arc<SleepNotifier>,
 
     /// Current state of the task.
     pub(crate) state: u8,
 
+    /// Latency matters or not
+    pub(crate) latency_matters: bool,
+
     /// Current reference count of the task.
     pub(crate) references: AtomicI16,
 
