@@ -109,7 +109,12 @@ where
     ///
     /// It is assumed that initially only the `Task` reference and the
     /// `JoinHandle` exist.
-    pub(crate) fn allocate(future: F, schedule: S, executor_id: usize) -> NonNull<()> {
+    pub(crate) fn allocate(
+        future: F,
+        schedule: S,
+        executor_id: usize,
+        latency_matters: bool,
+    ) -> NonNull<()> {
         // Compute the layout of the task for allocation. Abort if the computation
         // fails.
         let task_layout = abort_on_panic(Self::task_layout);
@@ -127,6 +132,7 @@ where
             (raw.header as *mut Header).write(Header {
                 notifier: sys::get_sleep_notifier_for(executor_id).unwrap(),
                 state: SCHEDULED | HANDLE,
+                latency_matters,
                 references: AtomicI16::new(0),
                 awaiter: None,
                 vtable: &TaskVTable {
@@ -220,7 +226,10 @@ where
         if Self::thread_id() != Some(raw.my_id()) {
             dbg_context!(ptr, "foreign", {
                 let notifier = raw.notifier();
-                notifier.queue_waker(Waker::from_raw(Self::clone_waker(ptr)));
+                notifier.queue_waker(
+                    Waker::from_raw(Self::clone_waker(ptr)),
+                    (*raw.header).latency_matters,
+                );
             });
         } else {
             let state = (*raw.header).state;
@@ -281,7 +290,7 @@ where
 
     /// Drops a waker.
     ///
-    /// This function will decrement the reference count. If it drops down to
+    /// This function will decrement the reference count. If it drops to
     /// zero, the associated join handle has been dropped too, and the task
     /// has not been completed, then it will get scheduled one more time so
     /// that its future gets dropped by the executor.
@@ -297,7 +306,10 @@ where
                     // will be destroyed
                     if Self::decrement_references(&mut *(raw.header as *mut Header)) == 0 {
                         let notifier = raw.notifier();
-                        notifier.queue_waker(Waker::from_raw(Self::clone_waker(ptr)));
+                        notifier.queue_waker(
+                            Waker::from_raw(Self::clone_waker(ptr)),
+                            (*raw.header).latency_matters,
+                        );
                     }
                     return;
                 });
@@ -328,7 +340,7 @@ where
 
     /// Drops a task.
     ///
-    /// This function will decrement the reference count. If it drops down to
+    /// This function will decrement the reference count. If it drops to
     /// zero and the associated join handle has been dropped too, then the
     /// task gets destroyed.
     #[inline]
@@ -358,16 +370,15 @@ where
             let raw = Self::from_ptr(ptr);
             Self::increment_references(&mut *(raw.header as *mut Header));
 
-            let guard;
             // Calling of schedule functions itself does not increment references,
             // if the schedule function has captured variables, increment references
             // so if task being dropped inside schedule function , function itself
             // will keep valid data till the end of execution.
-            if mem::size_of::<S>() > 0 {
-                guard = Some(Waker::from_raw(Self::clone_waker(ptr)));
+            let guard = if mem::size_of::<S>() > 0 {
+                Some(Waker::from_raw(Self::clone_waker(ptr)))
             } else {
-                guard = None;
-            }
+                None
+            };
 
             let task = Task {
                 raw_task: NonNull::new_unchecked(ptr as *mut ()),
@@ -550,10 +561,9 @@ where
                     (*(raw.header as *mut Header)).state =
                         ((*(raw.header)).state & !RUNNING & !SCHEDULED) | CLOSED;
 
-                    // drop tasks future, and drop the task reference.
-
+                    // Drop tasks future, and drop the task reference.
                     // The thread that closed the task didn't drop the future because it
-                    // was running so now it's our responsibility to do so.
+                    // was running, so now it's our responsibility to do so.
                     RawTask::<F, R, S>::drop_future(ptr);
 
                     // Notify the awaiter that the future has been dropped.
